@@ -22,34 +22,35 @@
  *     copyright remains in the source code and all documentation
  ****************************************************************************/
 
+/* IMPORTANT NOTE:
+ *
+ * The functions in this module *must not* assume that the geometries in the
+ * FvwmWindow structure reflect the desired geometry of the window or its
+ * parts.  While the window is resized or shaded, they may hold the old
+ * geometry instead of the new one (but you can not rely on this).  Therefore,
+ * these geometries must not be accessed directly or indirectly (by the
+ * functions from geometry,c).  Use the geometries that are passed in via
+ * structure pointers, e.d. "td".
+ */
+
 /* ---------------------------- included header files ----------------------- */
 
 #include "config.h"
 
 #include <stdio.h>
-#include <signal.h>
 
 #include "libs/fvwmlib.h"
-#include "libs/FShape.h"
-#include "libs/Flocale.h"
-#include <libs/gravity.h>
+#include "libs/Picture.h"
+#include "libs/PictureGraphics.h"
 #include "fvwm.h"
+#include "execcontext.h"
 #include "externs.h"
-#include "events.h"
-#include "cursor.h"
-#include "functions.h"
-#include "bindings.h"
 #include "misc.h"
 #include "screen.h"
-#include "defaults.h"
 #include "geometry.h"
 #include "borders.h"
 #include "builtins.h"
 #include "icons.h"
-#include "module_interface.h"
-#include "colorset.h"
-#include "libs/PictureGraphics.h"
-#include "add_window.h"
 #include "frame.h"
 
 /* ---------------------------- local definitions --------------------------- */
@@ -78,7 +79,8 @@ typedef struct
 		Pixmap p;
 		Pixmap shape;
 		Pixmap alpha;
-		Pixmap depth;
+		int depth;
+		FvwmRenderAttributes fra;
 		size_rect size;
 		struct
 		{
@@ -206,22 +208,28 @@ static void border_render_into_pixmap(
 	GC gc, FvwmPicture *src, Pixmap dest, int x_start, int y_start,
 	int width, int height, Bool stretch)
 {
-	Pixmap pm;
+	FvwmPicture *pm = NULL;
+	Bool do_repeat = False;
 
 	if (stretch)
 	{
-		pm = CreateStretchPixmap(
-			dpy, src->picture, src->width, src->height, src->depth,
-			width, height, gc);
+		pm = PGraphicsCreateStretchPicture(
+			dpy, Scr.NoFocusWin, src, width, height,
+			gc, Scr.MonoGC, None);
 	}
 	else
 	{
-		pm = CreateTiledPixmap(
-			dpy, src->picture, src->width, src->height, width,
-			height, src->depth, gc);
+		pm = src;
+		do_repeat = True;
+	}
+	PGraphicsRenderPicture(
+		dpy, Scr.NoFocusWin, pm, 0, dest, gc, Scr.MonoGC, None,
+		0, 0, pm->width, pm->height,
+		x_start, y_start, width, height, do_repeat);
+	if (pm != src)
+	{
+		PDestroyFvwmPicture(dpy, pm);
 	}
-	XCopyArea(dpy, pm, dest, gc, 0, 0, width, height, x_start, y_start);
-	XFreePixmap(dpy, pm);
 
 	return;
 }
@@ -249,7 +257,7 @@ static int get_multipm_length(
  *
  ****************************************************************************/
 static void border_draw_multi_pixmap_titlebar(
-	FvwmWindow *fw, DecorFace *df, Pixmap dest_pix)
+	FvwmWindow *fw, titlebar_descr *td, DecorFace *df, Pixmap dest_pix)
 {
 	GC gc;
 	char *title;
@@ -267,21 +275,21 @@ static void border_draw_multi_pixmap_titlebar(
 	gc = Scr.TitleGC;
 	XSetClipMask(dpy, gc, None);
 	title = fw->visible_name;
-
-	tmp_g.width = 0;
-	tmp_g.height = 0;
-	get_title_geometry(fw, &tmp_g);
+	tmp_g.width = td->layout.title_g.width;
+	tmp_g.height = td->layout.title_g.height;
 	if (pm[TBP_MAIN])
 	{
 		border_render_into_pixmap(
-			gc, pm[TBP_MAIN], dest_pix, 0, 0, tmp_g.width,
-			tmp_g.height, (stretch_flags & (1 << TBP_MAIN)));
+			gc, pm[TBP_MAIN], dest_pix, 0, 0,
+			td->layout.title_g.width, td->layout.title_g.height,
+			(stretch_flags & (1 << TBP_MAIN)));
 	}
 	else if (!title)
 	{
 		border_render_into_pixmap(
-			gc, pm[TBP_LEFT_MAIN], dest_pix, 0, 0, tmp_g.width,
-			tmp_g.height, (stretch_flags & (1 << TBP_LEFT_MAIN)));
+			gc, pm[TBP_LEFT_MAIN], dest_pix, 0, 0,
+			td->layout.title_g.width, td->layout.title_g.height,
+			(stretch_flags & (1 << TBP_LEFT_MAIN)));
 	}
 
 	if (title)
@@ -478,35 +486,30 @@ static Bool is_button_toggled(
 static ButtonState border_flags_to_button_state(
 	int is_pressed, int is_lit, int is_toggled)
 {
-	if (Scr.gs.use_active_down_buttons)
+	if (!is_lit && Scr.gs.use_inactive_buttons)
 	{
-		if (Scr.gs.use_inactive_buttons && !is_lit)
+		if (is_pressed && Scr.gs.use_inactive_down_buttons)
 		{
-			return (is_toggled) ? BS_ToggledInactive : BS_Inactive;
+			return (is_toggled) ?
+				BS_ToggledInactiveDown : BS_InactiveDown;
 		}
 		else
 		{
-			if (is_pressed)
-			{
-				return (is_toggled) ?
-					BS_ToggledActiveDown : BS_ActiveDown;
-			}
-			else
-			{
-				return (is_toggled) ?
-					BS_ToggledActiveUp : BS_ActiveUp;
-			}
+			return (is_toggled) ?
+				BS_ToggledInactiveUp : BS_InactiveUp;
 		}
 	}
 	else
 	{
-		if (Scr.gs.use_inactive_buttons && !is_lit)
+		if (is_pressed && Scr.gs.use_active_down_buttons)
 		{
-			return (is_toggled) ? BS_ToggledInactive : BS_Inactive;
+			return (is_toggled) ?
+				BS_ToggledActiveDown : BS_ActiveDown;
 		}
 		else
 		{
-			return (is_toggled) ? BS_ToggledActiveUp : BS_ActiveUp;
+			return (is_toggled) ?
+				BS_ToggledActiveUp : BS_ActiveUp;
 		}
 	}
 }
@@ -1361,14 +1364,13 @@ static void border_fill_pixmap_background(
 		xgcv.background = cd->back_color;
 		valuemask = GCForeground|GCBackground;
 		XChangeGC(dpy, Scr.TileGC, valuemask, &xgcv);
-		PGraphicsCopyPixmaps(dpy, bg->pixmap.p, bg->pixmap.shape,
-				     bg->pixmap.alpha, bg->pixmap.depth,
-				     dest_pix,
-				     Scr.TileGC,
-				     0, 0,
-				     bg->pixmap.size.width,
-				     bg->pixmap.size.height,
-				     pixmap_g->x, pixmap_g->y);
+		PGraphicsRenderPixmaps(
+			dpy, Scr.NoFocusWin, bg->pixmap.p, bg->pixmap.shape,
+			bg->pixmap.alpha, bg->pixmap.depth, &(bg->pixmap.fra),
+			dest_pix, Scr.TileGC, Scr.MonoGC, None,
+			0, 0, bg->pixmap.size.width, bg->pixmap.size.height,
+			pixmap_g->x, pixmap_g->y, pixmap_g->width - pixmap_g->x,
+			pixmap_g->height - pixmap_g->y, False);
 	}
 	else
 	{
@@ -1377,17 +1379,14 @@ static void border_fill_pixmap_background(
 		xgcv.background = cd->back_color;
 		valuemask = GCForeground|GCBackground;
 		XChangeGC(dpy, Scr.TileGC, valuemask, &xgcv);
-		PGraphicsTileRectangle(dpy, Scr.Root,
-				       bg->pixmap.p,
-				       bg->pixmap.shape,
-				       bg->pixmap.alpha,
-				       bg->pixmap.depth,
-				       0, 0,
-				       dest_pix, Scr.TileGC, Scr.MonoGC,
-				       pixmap_g->x, pixmap_g->y,
-				       pixmap_g->width - pixmap_g->x,
-				       pixmap_g->height - pixmap_g->y);
-
+		PGraphicsRenderPixmaps(
+			dpy, Scr.NoFocusWin, bg->pixmap.p, bg->pixmap.shape,
+			bg->pixmap.alpha, bg->pixmap.depth, &(bg->pixmap.fra),
+			dest_pix, Scr.TileGC, Scr.MonoGC, None,
+			0, 0, bg->pixmap.size.width, bg->pixmap.size.height,
+			pixmap_g->x, pixmap_g->y,
+			pixmap_g->width - pixmap_g->x,
+			pixmap_g->height - pixmap_g->y, True);
 	}
 
 	return;
@@ -1404,6 +1403,7 @@ static void border_get_border_background(
 		bg->pixmap.alpha = None;
 		bg->pixmap.depth = Pdepth;
 		bg->pixmap.flags.is_tiled = 1;
+		bg->pixmap.fra.mask = 0;
 	}
 	else
 	{
@@ -1563,6 +1563,7 @@ static void border_draw_decor_to_pixmap(
 	FvwmPicture *p;
 	int border;
 
+	bg.pixmap.fra.mask = 0;
 	switch (type)
 	{
 	case SimpleButton:
@@ -1586,6 +1587,11 @@ static void border_draw_decor_to_pixmap(
 				break;
 			}
 			p = fw->mini_icon;
+			if (cd->cs >= 0)
+			{
+				bg.pixmap.fra.mask |= FRAM_HAVE_ICON_CSET;
+				bg.pixmap.fra.colorset = &Colorset[cd->cs];
+			}
 		}
 		else
 		{
@@ -2112,7 +2118,7 @@ static void border_set_title_pixmap(
 #ifdef FANCY_TITLEBARS
 	else if (tdd.df->style.face_type == MultiPixmap)
 	{
-		border_draw_multi_pixmap_titlebar(fw, tdd.df, dest_pix);
+		border_draw_multi_pixmap_titlebar(fw, td, tdd.df, dest_pix);
 	}
 #endif
 	else
@@ -2600,6 +2606,13 @@ void border_draw_decorations(
 	{
 		return;
 	}
+	if (WAS_NEVER_DRAWN(fw))
+	{
+		/* force drawing everything */
+		do_force = True;
+		draw_parts = PART_ALL;
+		SET_WAS_NEVER_DRAWN(fw, 0);
+	}
 	memset(&cd, 0, sizeof(cd));
 	/* can't compare with True here, old code calls this with value "2" */
 	if (do_force != False)
@@ -2636,7 +2649,7 @@ void border_draw_decorations(
 	}
 	if (IS_ICONIFIED(fw))
 	{
-		DrawIconWindow(fw);
+		DrawIconWindow(fw, True, True, True, NULL);
 		return;
 	}
 	/* calculate some values and flags */
@@ -2727,6 +2740,8 @@ void CMD_ButtonState(F_CMD_ARGS)
 				DEFAULT_USE_ACTIVE_DOWN_BUTTONS;
 			Scr.gs.use_inactive_buttons =
 				DEFAULT_USE_INACTIVE_BUTTONS;
+			Scr.gs.use_inactive_down_buttons =
+				DEFAULT_USE_INACTIVE_DOWN_BUTTONS;
 			return;
 		}
 		first = False;
@@ -2740,7 +2755,13 @@ void CMD_ButtonState(F_CMD_ARGS)
 		{
 			Scr.gs.use_inactive_buttons = ParseToggleArgument(
 				action, &action,
-				DEFAULT_USE_ACTIVE_DOWN_BUTTONS, True);
+				DEFAULT_USE_INACTIVE_BUTTONS, True);
+		}
+		else if (StrEquals("inactivedown", token))
+		{
+			Scr.gs.use_inactive_down_buttons = ParseToggleArgument(
+				action, &action,
+				DEFAULT_USE_INACTIVE_DOWN_BUTTONS, True);
 		}
 		else
 		{
@@ -2748,8 +2769,10 @@ void CMD_ButtonState(F_CMD_ARGS)
 				DEFAULT_USE_ACTIVE_DOWN_BUTTONS;
 			Scr.gs.use_inactive_buttons =
 				DEFAULT_USE_INACTIVE_BUTTONS;
+			Scr.gs.use_inactive_down_buttons =
+				DEFAULT_USE_INACTIVE_DOWN_BUTTONS;
 			fvwm_msg(ERR, "cmd_button_state",
-				 "unknown button state %s\n", token);
+				 "Unknown button state %s", token);
 			return;
 		}
 	}
