@@ -27,18 +27,14 @@
 #include "config.h"
 
 #include <stdio.h>
-#include <signal.h>
 #include <errno.h>
 #include <X11/keysym.h>
 
 #include "libs/fvwmlib.h"
 #include "libs/fvwmsignal.h"
 #include "libs/setpgrp.h"
-#include "libs/FShape.h"
-#include "libs/Flocale.h"
-#include <libs/gravity.h>
-#include <libs/Picture.h>
-#include <libs/PictureUtils.h>
+#include "libs/Picture.h"
+#include "libs/PictureUtils.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "colorset.h"
@@ -48,25 +44,19 @@
 #include "functions.h"
 #include "commands.h"
 #include "screen.h"
-#include "defaults.h"
 #include "builtins.h"
 #include "module_interface.h"
 #include "borders.h"
 #include "frame.h"
-#include "geometry.h"
 #include "events.h"
-#include "gnome.h"
 #include "ewmh.h"
-#include "icons.h"
-#include "menus.h"
 #include "virtual.h"
 #include "decorations.h"
 #include "add_window.h"
 #include "update.h"
 #include "style.h"
-#include "focus.h"
-#include "stack.h"
 #include "move_resize.h"
+#include "menus.h"
 #ifdef HAVE_STROKE
 #include "stroke.h"
 #endif /* HAVE_STROKE */
@@ -100,14 +90,26 @@ typedef struct
 static char *exec_shell_name="/bin/sh";
 
 /* button state strings must match the enumerated states */
-static char *button_states[BS_MaxButtonState + 1] =
+static char *button_states[BS_MaxButtonStateName + 1] =
 {
 	"ActiveUp",
 	"ActiveDown",
-	"Inactive",
+	"InactiveUp",
+	"InactiveDown",
 	"ToggledActiveUp",
 	"ToggledActiveDown",
+	"ToggledInactiveUp",
+	"ToggledInactiveDown",
+	"Active",
+	"Inactive",
+	"ToggledActive",
 	"ToggledInactive",
+	"AllNormal",
+	"AllToggled",
+	"AllActive",
+	"AllInactive",
+	"AllUp",
+	"AllDown",
 	NULL
 };
 
@@ -196,9 +198,13 @@ static char *ReadTitleButton(
 	char *t;
 	int i;
 	int bs;
+	int bs_start, bs_end;
 	int pstyle = 0;
 	DecorFace tmpdf;
-	Bool do_set_all = False;
+
+	Bool multiple;
+	int use_mask = 0;
+	int set_mask = 0;
 
 	s = SkipSpaces(s, NULL, 0);
 	t = GetNextTokenIndex(s, button_states, 0, &bs);
@@ -206,9 +212,79 @@ static char *ReadTitleButton(
 	{
 		s = SkipSpaces(t, NULL, 0);
 	}
+
+	if (bs == BS_All)
+	{
+		use_mask = 0;
+		set_mask = 0;
+	}
+	else if (bs == BS_Active)
+	{
+		use_mask = BS_MASK_INACTIVE | BS_MASK_TOGGLED;
+		set_mask = 0;
+	}
+	else if (bs == BS_Inactive)
+	{
+		use_mask = BS_MASK_INACTIVE | BS_MASK_TOGGLED;
+		set_mask = BS_MASK_INACTIVE;
+	}
+	else if (bs == BS_ToggledActive)
+	{
+		use_mask = BS_MASK_INACTIVE | BS_MASK_TOGGLED;
+		set_mask = BS_MASK_TOGGLED;
+	}
+	else if (bs == BS_ToggledInactive)
+	{
+		use_mask = BS_MASK_INACTIVE | BS_MASK_TOGGLED;
+		set_mask = BS_MASK_INACTIVE | BS_MASK_TOGGLED;
+	}
+	else if (bs == BS_AllNormal)
+	{
+		use_mask = BS_MASK_TOGGLED;
+		set_mask = 0;
+	}
+	else if (bs == BS_AllToggled)
+	{
+		use_mask = BS_MASK_TOGGLED;
+		set_mask = BS_MASK_TOGGLED;
+	}
+	else if (bs == BS_AllActive)
+	{
+		use_mask = BS_MASK_INACTIVE;
+		set_mask = 0;
+	}
+	else if (bs == BS_AllInactive)
+	{
+		use_mask = BS_MASK_INACTIVE;
+		set_mask = BS_MASK_INACTIVE;
+	}
+	else if (bs == BS_AllUp)
+	{
+		use_mask = BS_MASK_DOWN;
+		set_mask = 0;
+	}
+	else if (bs == BS_AllDown)
+	{
+		use_mask = BS_MASK_DOWN;
+		set_mask = BS_MASK_DOWN;
+	}
+
+	if ((bs & BS_MaxButtonStateMask) == bs)
+	{
+		multiple = False;
+		bs_start = bs;
+		bs_end = bs;
+	}
 	else
 	{
-		do_set_all = True;
+		multiple = True;
+		bs_start = 0;
+		bs_end = BS_MaxButtonState - 1;
+		for (i = bs_start; (i & use_mask) != set_mask && i <= bs_end;
+			i++)
+		{
+			bs_start++;
+		}
 	}
 
 	if (*s == '(')
@@ -217,8 +293,9 @@ static char *ReadTitleButton(
 		pstyle = 1;
 		if (!(end = strchr(++s, ')')))
 		{
-			fvwm_msg(ERR, "ReadTitleButton",
-				 "missing parenthesis: %s", s);
+			fvwm_msg(
+				ERR, "ReadTitleButton",
+				"missing parenthesis: %s", s);
 			return NULL;
 		}
 		s = SkipSpaces(s, NULL, 0);
@@ -237,28 +314,25 @@ static char *ReadTitleButton(
 	memset(&tmpdf, 0, sizeof(DecorFace));
 	DFS_FACE_TYPE(tmpdf.style) = SimpleButton;
 
-	if (strncmp(spec, "--",2)==0)
+	if (strncmp(spec, "--", 2) == 0)
 	{
 		/* only change flags */
-		if (do_set_all)
+		Bool verbose = True;
+		for (i = bs_start; i <= bs_end; ++i)
 		{
-			for (i = 0; i < BS_MaxButtonState; ++i)
+			if (multiple && (i & use_mask) != set_mask)
 			{
-				ReadDecorFace(
-					spec, &TB_STATE(*tb)[i], BS_All, !i);
+				continue;
 			}
-		}
-		else
-		{
-			ReadDecorFace(spec, &TB_STATE(*tb)[bs], button, True);
+			ReadDecorFace(spec, &TB_STATE(*tb)[i], button, verbose);
+			verbose = False;
 		}
 	}
 	else if (ReadDecorFace(spec, &tmpdf, button, True))
 	{
-		int b = (do_set_all) ? 0 : bs;
 		if (append)
 		{
-			DecorFace *head = &TB_STATE(*tb)[b];
+			DecorFace *head = &TB_STATE(*tb)[bs_start];
 			DecorFace *tail = head;
 			DecorFace *next;
 
@@ -269,12 +343,13 @@ static char *ReadTitleButton(
 			tail->next = (DecorFace *)safemalloc(sizeof(DecorFace));
 			memcpy(tail->next, &tmpdf, sizeof(DecorFace));
 			if (DFS_FACE_TYPE(tail->next->style) == VectorButton &&
-			    DFS_FACE_TYPE((&TB_STATE(*tb)[b])->style) ==
+			    DFS_FACE_TYPE((&TB_STATE(*tb)[bs_start])->style) ==
 			    DefaultVectorButton)
 			{
 				/* override the default vector style */
-				memcpy(&tail->next->style, &head->style,
-				       sizeof(DecorFaceStyle));
+				memcpy(
+					&tail->next->style, &head->style,
+					sizeof(DecorFaceStyle));
 				DFS_FACE_TYPE(tail->next->style) = VectorButton;
 				next = head->next;
 				head->next = NULL;
@@ -282,68 +357,61 @@ static char *ReadTitleButton(
 				memcpy(head, next, sizeof(DecorFace));
 				free(next);
 			}
-			if (do_set_all)
+			for (i = bs_start + 1; i <= bs_end; ++i)
 			{
-				for (i = 1; i < BS_MaxButtonState; ++i)
+				if (multiple && (i & use_mask) != set_mask)
 				{
-					if (i == b)
-					{
-						/* already done above */
-						continue;
-					}
-					head = &TB_STATE(*tb)[i];
-					tail = head;
-					while (tail->next)
-					{
-						tail = tail->next;
-					}
-					tail->next = (DecorFace *)safemalloc(
-						sizeof(DecorFace));
-					memset(&DFS_FLAGS(tail->next->style), 0,
-					       sizeof(DFS_FLAGS(tail->next->
-								style)));
+					continue;
+				}
+				head = &TB_STATE(*tb)[i];
+				tail = head;
+				while (tail->next)
+				{
+					tail = tail->next;
+				}
+				tail->next = (DecorFace *)safemalloc(
+					sizeof(DecorFace));
+				memset(
+					&DFS_FLAGS(tail->next->style), 0,
+					sizeof(DFS_FLAGS(tail->next->style)));
+				DFS_FACE_TYPE(tail->next->style) =
+					SimpleButton;
+				tail->next->next = NULL;
+				ReadDecorFace(spec, tail->next, button, False);
+				if (DFS_FACE_TYPE(tail->next->style) ==
+				    VectorButton &&
+				    DFS_FACE_TYPE((&TB_STATE(*tb)[i])->style) ==
+				    DefaultVectorButton)
+				{
+					/* override the default vector style */
+					memcpy(
+						&tail->next->style,
+						&head->style,
+						sizeof(DecorFaceStyle));
 					DFS_FACE_TYPE(tail->next->style) =
-						SimpleButton;
-					tail->next->next = NULL;
-					ReadDecorFace(
-						spec, tail->next, button,
-						False);
-					if (DFS_FACE_TYPE(tail->next->style) ==
-					    VectorButton &&
-					    DFS_FACE_TYPE(
-						    (&TB_STATE(*tb)[i])->style)
-					    == DefaultVectorButton)
-					{
-						/* override the default vector
-						 * style */
-						memcpy(&tail->next->style,
-						       &head->style,
-						       sizeof(DecorFaceStyle));
-						DFS_FACE_TYPE(
-							tail->next->style) =
-							VectorButton;
-						next = head->next;
-						head->next = NULL;
-						FreeDecorFace(dpy, head);
-						memcpy(head, next,
-						       sizeof(DecorFace));
-						free(next);
-					}
+						VectorButton;
+					next = head->next;
+					head->next = NULL;
+					FreeDecorFace(dpy, head);
+					memcpy(head, next, sizeof(DecorFace));
+					free(next);
 				}
 			}
 		}
 		else
 		{
-			FreeDecorFace(dpy, &TB_STATE(*tb)[b]);
-			memcpy(&(TB_STATE(*tb)[b]), &tmpdf, sizeof(DecorFace));
-			if (do_set_all)
+			FreeDecorFace(dpy, &TB_STATE(*tb)[bs_start]);
+			memcpy(
+				&(TB_STATE(*tb)[bs_start]), &tmpdf,
+				sizeof(DecorFace));
+			for (i = bs_start + 1; i <= bs_end; ++i)
 			{
-				for (i = 1; i < BS_MaxButtonState; ++i)
+				if (multiple && (i & use_mask) != set_mask)
 				{
-					ReadDecorFace(
-						spec, &TB_STATE(*tb)[i],
-						button, False);
+					continue;
 				}
+				ReadDecorFace(
+					spec, &TB_STATE(*tb)[i], button, False);
 			}
 		}
 	}
@@ -357,6 +425,34 @@ static char *ReadTitleButton(
 	return end;
 }
 
+/* Remove the given decor from all windows */
+static void __remove_window_decors(F_CMD_ARGS, FvwmDecor *d)
+{
+	const exec_context_t *exc2;
+	exec_context_changes_t ecc;
+	FvwmWindow *t;
+
+	for (t = Scr.FvwmRoot.next; t; t = t->next)
+	{
+		if (t->decor == d)
+		{
+			/* remove the extra title height now because we delete
+			 * the current decor before calling ChangeDecor(). */
+			t->frame_g.height -= t->decor->title_height;
+			t->decor = NULL;
+			ecc.w.fw = t;
+			ecc.w.wcontext = C_WINDOW;
+			exc2 = exc_clone_context(
+				exc, &ecc, ECC_FW | ECC_WCONTEXT);
+			execute_function(
+				cond_rc, exc2, "ChangeDecor Default", 0);
+			exc_destroy_context(exc2);
+		}
+	}
+
+	return;
+}
+
 static void do_title_style(F_CMD_ARGS, Bool do_add)
 {
 	char *parm;
@@ -985,7 +1081,6 @@ void ApplyDefaultFontAndColors(void)
 	{
 		resize_geometry_window();
 	}
-
 	UpdateAllMenuStyles();
 
 	return;
@@ -1595,22 +1690,22 @@ Bool ReadDecorFace(char *s, DecorFace *df, int button, int verbose)
  * Diverts a style definition to an FvwmDecor structure (veliaa@rpi.edu)
  *
  ****************************************************************************/
-void AddToDecor(FvwmDecor *decor, char *s)
+void AddToDecor(F_CMD_ARGS, FvwmDecor *decor)
 {
-	if (!s)
+	if (!action)
 	{
 		return;
 	}
-	while (*s && isspace((unsigned char)*s))
+	while (*action && isspace((unsigned char)*action))
 	{
-		++s;
+		++action;
 	}
-	if (!*s)
+	if (!*action)
 	{
 		return;
 	}
 	Scr.cur_decor = decor;
-	old_execute_function(NULL, s, NULL, &Event, C_ROOT, -1, 0, NULL);
+	execute_function(cond_rc, exc, action, 0);
 	Scr.cur_decor = NULL;
 
 	return;
@@ -1706,7 +1801,7 @@ void CMD_CursorMove(F_CMD_ARGS)
 		fvwm_msg(ERR, "movecursor", "CursorMove needs 2 arguments");
 		return;
 	}
-	if (XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
+	if (FQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
 			  &x, &y, &JunkX, &JunkY, &JunkMask) == False)
 	{
 		/* pointer is on a different screen */
@@ -1779,18 +1874,15 @@ void CMD_CursorMove(F_CMD_ARGS)
 		y = pan_y;
 	}
 
-	XWarpPointer(dpy, None, Scr.Root, 0, 0, Scr.MyDisplayWidth,
+	FWarpPointer(dpy, None, Scr.Root, 0, 0, Scr.MyDisplayWidth,
 		     Scr.MyDisplayHeight, x, y);
 	return;
 }
 
 void CMD_Delete(F_CMD_ARGS)
 {
-	if (DeferExecution(
-		    eventp, &w, &fw, &context, CRS_DESTROY, ButtonRelease))
-	{
-		return;
-	}
+	FvwmWindow * const fw = exc->w.fw;
+
 	if (!is_function_allowed(F_DELETE, NULL, fw, True, True))
 	{
 		XBell(dpy, 0);
@@ -1824,11 +1916,8 @@ void CMD_Delete(F_CMD_ARGS)
 
 void CMD_Destroy(F_CMD_ARGS)
 {
-	if (DeferExecution(
-		    eventp, &w, &fw, &context, CRS_DESTROY, ButtonRelease))
-	{
-		return;
-	}
+	FvwmWindow * const fw = exc->w.fw;
+
 	if (IS_TEAR_OFF_MENU(fw))
 	{
 		CMD_Delete(F_PASS_ARGS);
@@ -1855,11 +1944,8 @@ void CMD_Destroy(F_CMD_ARGS)
 
 void CMD_Close(F_CMD_ARGS)
 {
-	if (DeferExecution(eventp, &w, &fw, &context, CRS_DESTROY,
-			   ButtonRelease))
-	{
-		return;
-	}
+	FvwmWindow * const fw = exc->w.fw;
+
 	if (IS_TEAR_OFF_MENU(fw))
 	{
 		CMD_Delete(F_PASS_ARGS);
@@ -1991,13 +2077,11 @@ void CMD_Refresh(F_CMD_ARGS)
 
 void CMD_RefreshWindow(F_CMD_ARGS)
 {
-	if (DeferExecution(eventp,&w,&fw,&context,CRS_SELECT,ButtonRelease))
-	{
-		return;
-	}
+	FvwmWindow * const fw = exc->w.fw;
+
 	refresh_window(
-		(context == C_ICON) ? FW_W_ICON_TITLE(fw) : FW_W_FRAME(fw),
-		True);
+		(exc->w.wcontext == C_ICON) ?
+		FW_W_ICON_TITLE(fw) : FW_W_FRAME(fw), True);
 
 	return;
 }
@@ -2009,9 +2093,8 @@ void CMD_Wait(F_CMD_ARGS)
 	Bool is_ungrabbed;
 	char *escape;
 	Window nonewin = None;
-	extern FvwmWindow *Fw;
 	char *wait_string, *rest;
-	FvwmWindow *s_Fw = Fw;
+	FvwmWindow *t;
 
 	/* try to get a single token */
 	rest = GetNextToken(action, &wait_string);
@@ -2050,48 +2133,55 @@ void CMD_Wait(F_CMD_ARGS)
 	is_ungrabbed = UngrabEm(GRAB_NORMAL);
 	while (!done && !isTerminated)
 	{
+		XEvent e;
 		if (BUSY_WAIT & Scr.BusyCursor)
 		{
 			XDefineCursor(dpy, Scr.Root, Scr.FvwmCursors[CRS_WAIT]);
 			redefine_cursor = True;
 		}
-		if (My_XNextEvent(dpy, &Event))
+		if (My_XNextEvent(dpy, &e))
 		{
-			DispatchEvent(False);
-			if (Event.type == MapNotify)
+			dispatch_event(&e);
+			if (XFindContext(
+				    dpy, e.xmap.window, FvwmContext,
+				    (caddr_t *)&t) == XCNOENT)
+			{
+				t = NULL;
+			}
+
+			if (e.type == MapNotify)
 			{
 				if (!*wait_string)
 				{
 					done = True;
 				}
-				if (Fw &&
-				    matchWildcards(wait_string, Fw->name.name)
-				    == True)
+				if (t && matchWildcards(
+					    wait_string, t->name.name) == True)
 				{
 					done = True;
 				}
-				else if (Fw && Fw->class.res_class &&
+				else if (t && t->class.res_class &&
 					 matchWildcards(
 						 wait_string,
-						 Fw->class.res_class) == True)
+						 t->class.res_class) == True)
 				{
 					done = True;
 				}
-				else if (Fw && Fw->class.res_name &&
+				else if (t && t->class.res_name &&
 					 matchWildcards(
 						 wait_string,
-						 Fw->class.res_name) == True)
+						 t->class.res_name) == True)
 				{
 					done = True;
 				}
 			}
-			else if (Event.type == KeyPress)
+			else if (e.type == KeyPress)
 			{
 				escape = CheckBinding(
 					Scr.AllBindings, STROKE_ARG(0)
-					Event.xkey.keycode,
-					Event.xkey.state, GetUnusedModifiers(),
-					GetContext(Fw,&Event,&nonewin),
+					e.xkey.keycode, e.xkey.state,
+					GetUnusedModifiers(),
+					GetContext(&t, t, &e, &nonewin),
 					KEY_BINDING);
 				if (escape != NULL)
 				{
@@ -2111,7 +2201,6 @@ void CMD_Wait(F_CMD_ARGS)
 	{
 		GrabEm(CRS_NONE, GRAB_NORMAL);
 	}
-	Fw = (check_if_fvwm_window_exists(s_Fw)) ? s_Fw : NULL;
 	free(wait_string);
 
 	return;
@@ -2156,6 +2245,29 @@ void CMD_Echo(F_CMD_ARGS)
 	return;
 }
 
+void CMD_PrintInfo(F_CMD_ARGS)
+{
+	int verbose;
+	char *rest, *subject;
+
+	rest = GetNextToken(action, &subject);
+	if (!rest || GetIntegerArguments(rest, NULL, &verbose, 1) != 1)
+	{
+		verbose = 0;
+	}
+	if (StrEquals(subject, "Colors"))
+	{
+		PicturePrintColorInfo(verbose);
+	}
+	else
+	{
+		fvwm_msg(ERR, "PrintInfo",
+			 "Unknown subject '%s'", action);
+	}
+
+	return;
+}
+
 void CMD_ColormapFocus(F_CMD_ARGS)
 {
 	if (MatchToken(action,"FollowsFocus"))
@@ -2352,7 +2464,6 @@ void CMD_Colorset(F_CMD_ARGS)
 		return;
 	}
 	parse_colorset(n, token);
-	BroadcastColorset(n);
 	if (n == Scr.DefaultColorset)
 	{
 		Scr.flags.do_need_window_update = 1;
@@ -2555,14 +2666,10 @@ void CMD_ChangeDecor(F_CMD_ARGS)
 {
 	char *item;
 	int old_height;
-	int extra_height;
 	FvwmDecor *decor = &Scr.DefaultDecor;
 	FvwmDecor *found = NULL;
+	FvwmWindow * const fw = exc->w.fw;
 
-	if (DeferExecution(eventp,&w,&fw,&context, CRS_SELECT,ButtonRelease))
-	{
-		return;
-	}
 	item = PeekToken(action, &action);
 	if (!action || !item)
 	{
@@ -2586,13 +2693,6 @@ void CMD_ChangeDecor(F_CMD_ARGS)
 	old_height = (fw->decor) ? fw->decor->title_height : 0;
 	fw->decor = found;
 	apply_decor_change(fw);
-	extra_height = (HAS_TITLE(fw) && old_height) ?
-		(old_height - fw->decor->title_height) : 0;
-	frame_force_setup_window(
-		fw, fw->frame_g.x, fw->frame_g.y, fw->frame_g.width,
-		fw->frame_g.height - extra_height, True);
-	border_draw_decorations(
-		fw, PART_ALL, (Scr.Hilite == fw), 2, CLEAR_ALL, NULL, NULL);
 
 	return;
 }
@@ -2637,26 +2737,9 @@ void CMD_DestroyDecor(F_CMD_ARGS)
 
 	if (found && (found != &Scr.DefaultDecor || do_recreate))
 	{
-		FvwmWindow *fw2;
-
 		if (!do_recreate)
 		{
-			for (fw2 = Scr.FvwmRoot.next; fw2; fw2 = fw2->next)
-			{
-				if (fw2->decor == found)
-				{
-					/* remove the extra title height now
-					 * because we delete the current decor
-					 * before calling ChangeDecor(). */
-					fw2->frame_g.height -=
-						fw2->decor->title_height;
-					fw2->decor = NULL;
-					old_execute_function(
-						NULL, "ChangeDecor Default",
-						fw2, eventp, C_WINDOW, *Module,
-						0, NULL);
-				}
-			}
+			__remove_window_decors(F_PASS_ARGS, found);
 		}
 		DestroyFvwmDecor(found);
 		if (do_recreate)
@@ -2691,16 +2774,16 @@ void CMD_DestroyDecor(F_CMD_ARGS)
  ****************************************************************************/
 void CMD_AddToDecor(F_CMD_ARGS)
 {
-	FvwmDecor *decor, *found = NULL;
-	char *item = NULL, *s = action;
-
-	s = GetNextToken(s, &item);
+	FvwmDecor *decor;
+	FvwmDecor *found = NULL;
+	char *item = NULL;
 
+	action = GetNextToken(action, &item);
 	if (!item)
 	{
 		return;
 	}
-	if (!s)
+	if (!action)
 	{
 		free(item);
 		return;
@@ -2735,7 +2818,7 @@ void CMD_AddToDecor(F_CMD_ARGS)
 
 	if (found)
 	{
-		AddToDecor(found, s);
+		AddToDecor(F_PASS_ARGS, found);
 		/* Set + state to last decor */
 		set_last_added_item(ADDED_DECOR, found);
 	}
@@ -3229,7 +3312,7 @@ void CMD_FakeClick(F_CMD_ARGS)
 	int maxdepth = 0;
 
 	/* get the mask of pressed/released buttons */
-	if (XQueryPointer(
+	if (FQueryPointer(
 		    dpy, Scr.Root, &root, &JunkRoot, &JunkX, &JunkY, &JunkX,
 		    &JunkY, &mask) == False)
 	{
@@ -3277,7 +3360,7 @@ void CMD_FakeClick(F_CMD_ARGS)
 				     depth++)
 				{
 					w = child_w;
-					if (XQueryPointer(
+					if (FQueryPointer(
 						    dpy, w, &root, &child_w,
 						    &rx, &ry, &x, &y,
 						    &JunkMask) == False)
@@ -3301,7 +3384,7 @@ void CMD_FakeClick(F_CMD_ARGS)
 				e.xbutton.window = w;
 				e.xbutton.subwindow = None;
 				e.xbutton.root = root;
-				e.xbutton.time = lastTimestamp;
+				e.xbutton.time = fev_get_evtime();
 				e.xbutton.x = x;
 				e.xbutton.y = y;
 				e.xbutton.x_root = rx;
@@ -3309,7 +3392,7 @@ void CMD_FakeClick(F_CMD_ARGS)
 				e.xbutton.button = val;
 				e.xbutton.state = mask;
 				e.xbutton.same_screen = (Scr.Root == root);
-				XSendEvent(
+				FSendEvent(
 					dpy, PointerWindow, True,
 					SubstructureNotifyMask | add_mask, &e);
 				XFlush(dpy);
@@ -3334,7 +3417,7 @@ void CMD_FakeClick(F_CMD_ARGS)
 			if (val > 0 && val <= 1000000)
 			{
 				usleep(1000 * val);
-				if (XQueryPointer(
+				if (FQueryPointer(
 					    dpy, Scr.Root, &root, &JunkRoot,
 					    &JunkX, &JunkY, &JunkX, &JunkY,
 					    &mask) == False)
@@ -3412,8 +3495,8 @@ void CMD_StrokeFunc(F_CMD_ARGS)
 {
 	int finished = 0;
 	int abort = 0;
-	int modifiers = eventp->xbutton.state;
-	int start_event_type = eventp->type;
+	int modifiers = exc->x.etrigger->xbutton.state;
+	int start_event_type = exc->x.etrigger->type;
 	char sequence[STROKE_MAX_SEQUENCE + 1];
 	char *stroke_action;
 	char *opt = NULL;
@@ -3423,30 +3506,28 @@ void CMD_StrokeFunc(F_CMD_ARGS)
 	Bool echo_sequence = False;
 	Bool draw_motion = False;
 	int i = 0;
-
-	int* x = (int*)0;
-	int* y = (int*)0;
+	int *x = NULL;
+	int *y = NULL;
 	const int STROKE_CHUNK_SIZE = 0xff;
 	int coords_size = STROKE_CHUNK_SIZE;
-
 	Window JunkRoot, JunkChild;
 	int JunkX, JunkY;
 	int tmpx, tmpy;
 	unsigned int JunkMask;
 	Bool feed_back = False;
 	int stroke_width = 1;
-
-	x = (int*)safemalloc(coords_size * sizeof(int));
-	y = (int*)safemalloc(coords_size * sizeof(int));
+	XEvent e;
 
 	if (!GrabEm(CRS_STROKE, GRAB_NORMAL))
 	{
 		XBell(dpy, 0);
 		return;
 	}
-
+	x = (int*)safemalloc(coords_size * sizeof(int));
+	y = (int*)safemalloc(coords_size * sizeof(int));
+	e = *exc->x.etrigger;
 	/* set the default option */
-	if (eventp->type == KeyPress || eventp->type == ButtonPress)
+	if (e.type == KeyPress || e.type == ButtonPress)
 	{
 		finish_on_release = True;
 	}
@@ -3539,7 +3620,7 @@ void CMD_StrokeFunc(F_CMD_ARGS)
 	if (draw_motion)
 	{
 		MyXGrabServer(dpy);
-		if (XQueryPointer(
+		if (FQueryPointer(
 			    dpy, Scr.Root, &JunkRoot, &JunkChild, &x[0], &y[0],
 			    &JunkX, &JunkY, &JunkMask) == False)
 		{
@@ -3554,23 +3635,20 @@ void CMD_StrokeFunc(F_CMD_ARGS)
 	while (!finished && !abort)
 	{
 		/* block until there is an event */
-		XMaskEvent(
+		FMaskEvent(
 			dpy,  ButtonPressMask | ButtonReleaseMask |
 			KeyPressMask | KeyReleaseMask | ButtonMotionMask |
-			PointerMotionMask, eventp);
-		/* Records the time */
-		StashEventTime(eventp);
-
-		switch (eventp->type)
+			PointerMotionMask, &e);
+		switch (e.type)
 		{
 		case MotionNotify:
-			if (eventp->xmotion.same_screen == False)
+			if (e.xmotion.same_screen == False)
 			{
 				continue;
 			}
-			if (eventp->xany.window != Scr.Root)
+			if (e.xany.window != Scr.Root)
 			{
-				if (XQueryPointer(
+				if (FQueryPointer(
 					    dpy, Scr.Root, &JunkRoot,
 					    &JunkChild, &tmpx, &tmpy, &JunkX,
 					    &JunkY, &JunkMask) == False)
@@ -3582,51 +3660,28 @@ void CMD_StrokeFunc(F_CMD_ARGS)
 			}
 			else
 			{
-				tmpx = eventp->xmotion.x;
-				tmpy = eventp->xmotion.y;
+				tmpx = e.xmotion.x;
+				tmpy = e.xmotion.y;
 			}
 			stroke_record(tmpx,tmpy);
-			if (draw_motion)
+			if (draw_motion && (x[i] != tmpx || y[i] != tmpy))
 			{
-				if ((x[i] != tmpx || y[i] != tmpy))
+				i++;
+				if (i >= coords_size)
 				{
-					i++;
-					if (i >= coords_size)
-					{
-						coords_size +=
-							STROKE_CHUNK_SIZE;
-						x = (int*)saferealloc(
-							(void *)x, coords_size *
-							sizeof(int));
-						y = (int*)saferealloc(
-							(void *)y, coords_size *
-							sizeof(int));
-						if (!x || !y)
-						{
-							/* unlikely */
-							fvwm_msg(WARN,
-								 "strokeFunc",
-								 "unable to"
-								 " allocate %d"
-								 " bytes ..."
-								 " aborted.",
-								 coords_size *
-								 sizeof(int));
-							abort = 1;
-							i = -1; /* no undrawing
-								 * possible
-								 * since
-								 * either x or
-								 * y == 0 */
-							break;
-						}
-					}
-					x[i] = tmpx;
-					y[i] = tmpy;
-					XDrawLine(
-						dpy, Scr.Root, Scr.XorGC,
-						x[i-1], y[i-1], x[i], y[i]);
+					coords_size += STROKE_CHUNK_SIZE;
+					x = (int*)saferealloc(
+						(void *)x, coords_size *
+						sizeof(int));
+					y = (int*)saferealloc(
+						(void *)y, coords_size *
+						sizeof(int));
 				}
+				x[i] = tmpx;
+				y[i] = tmpy;
+				XDrawLine(
+					dpy, Scr.Root, Scr.XorGC, x[i-1],
+					y[i-1], x[i], y[i]);
 			}
 			break;
 		case ButtonRelease:
@@ -3643,7 +3698,7 @@ void CMD_StrokeFunc(F_CMD_ARGS)
 			}
 			break;
 		case KeyPress:
-			keysym = XLookupKeysym(&eventp->xkey,0);
+			keysym = XLookupKeysym(&e.xkey, 0);
 			/* abort if Escape or Delete is pressed (as in menus.c)
 			 */
 			if (keysym == XK_Escape || keysym == XK_Delete ||
@@ -3680,6 +3735,9 @@ void CMD_StrokeFunc(F_CMD_ARGS)
 		}
 		XSetLineAttributes(dpy,Scr.XorGC,0,LineSolid,CapButt,JoinMiter);
 		MyXUngrabServer(dpy);
+	}
+	if (x != NULL)
+	{
 		free(x);
 		free(y);
 	}
@@ -3700,7 +3758,7 @@ void CMD_StrokeFunc(F_CMD_ARGS)
 	{
 		char num_seq[STROKE_MAX_SEQUENCE + 1];
 
-		for (i=0;sequence[i] != '\0';i++)
+		for (i = 0; sequence[i] != '\0';i++)
 		{
 			/* Telephone to numeric pad */
 			if ('7' <= sequence[i] && sequence[i] <= '9')
@@ -3729,19 +3787,24 @@ void CMD_StrokeFunc(F_CMD_ARGS)
 	/* check for a binding */
 	stroke_action = CheckBinding(
 		Scr.AllBindings, sequence, 0, modifiers, GetUnusedModifiers(),
-		context, STROKE_BINDING);
+		exc->w.wcontext, STROKE_BINDING);
 
 	/* execute the action */
 	if (stroke_action != NULL)
 	{
+		const exec_context_t *exc2;
+		exec_context_changes_t ecc;
+
 		if (feed_back && atoi(sequence) != 0)
 		{
 			GrabEm(CRS_WAIT, GRAB_BUSY);
 			usleep(200000);
 			UngrabEm(GRAB_BUSY);
 		}
-		old_execute_function(
-			NULL, stroke_action, fw, eventp, context, -1, 0, NULL);
+		ecc.x.etrigger = &e;
+		exc2 = exc_clone_context(exc, &ecc, ECC_ETRIGGER);
+		execute_function(cond_rc, exc2, stroke_action, 0);
+		exc_destroy_context(exc2);
 	}
 
 	return;
@@ -3753,12 +3816,8 @@ void CMD_State(F_CMD_ARGS)
 	unsigned int state;
 	int toggle;
 	int n;
+	FvwmWindow * const fw = exc->w.fw;
 
-	if (DeferExecution(
-		    eventp, &w, &fw, &context, CRS_SELECT, ButtonRelease))
-	{
-		return;
-	}
 	n = GetIntegerArguments(action, &action, &state, 1);
 	if (n <= 0)
 	{
