@@ -43,22 +43,15 @@
 /**    OR PERFORMANCE OF THIS SOFTWARE.                                     **/
 /*****************************************************************************/
 
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <X11/Intrinsic.h>
-
-extern void Done(int, char *) __attribute__((__noreturn__));
-extern void setInitFunctionName(int n, const char *name);
-extern const char *getInitFunctionName(int n);
-extern void CaptureAllWindows(Bool is_recapture);
+void Done(int, char *) __attribute__((__noreturn__));
+void setInitFunctionName(int n, const char *name);
+const char *getInitFunctionName(int n);
 
 extern char NoName[];
 extern char NoClass[];
 extern char NoResource[];
 extern XGCValues Globalgcv;
 extern unsigned long Globalgcm;
-extern Time lastTimestamp;
-extern XEvent Event;
 extern int master_pid;
 extern Display *dpy;
 extern XContext FvwmContext;
