@@ -19,18 +19,22 @@
 
 /* ---------------------------- included header files ----------------------- */
 
-/* ---------------------------- global definitions -------------------------- */
+#include "execcontext.h"
 
-/* for exec_flags parameter of ExecuteFunction */
-#define FUNC_DONT_EXPAND_COMMAND 0x01
+/* ---------------------------- global definitions -------------------------- */
 
 /* Bits for the function flag byte. */
 enum
 {
-	FUNC_NEEDS_WINDOW = 0x01,
-	FUNC_DONT_REPEAT  = 0x02,
-	FUNC_ADD_TO       = 0x04,
-	FUNC_DECOR        = 0x08
+	FUNC_NEEDS_WINDOW        = 0x01,
+	FUNC_DONT_REPEAT         = 0x02,
+	FUNC_ADD_TO              = 0x04,
+	FUNC_DECOR               = 0x08,
+	FUNC_ALLOW_UNMANAGED     = 0x10,
+	/* only to be passed to execute_function() */
+	FUNC_IS_UNMANAGED        = 0x20,
+	FUNC_DONT_EXPAND_COMMAND = 0x40,
+	FUNC_DONT_DEFER          = 0x80
 };
 
 /* ---------------------------- global macros ------------------------------- */
@@ -48,6 +52,8 @@ typedef struct
 #endif
 	short func_type;
 	FUNC_FLAGS_TYPE flags;
+	int cursor;
+	int evtype;
 } func_type;
 
 /* ---------------------------- exported variables (globals) ---------------- */
@@ -58,13 +64,14 @@ void find_func_type(
 	char *action, short *func_type, unsigned char *flags);
 Bool functions_is_complex_function(
 	const char *function_name);
-int DeferExecution(
-	XEvent *, Window *,FvwmWindow **, unsigned long *, cursor_t, int);
 void execute_function(
-	exec_func_args_type *efa);
-void old_execute_function(
-	fvwm_cond_func_rc *cond_rc, char *action, FvwmWindow *fw,
-	XEvent *eventp, unsigned long context, int Module,
-	FUNC_FLAGS_TYPE exec_flags, char *args[]);
+	fvwm_cond_func_rc *cond_rc, const exec_context_t *exc, char *action,
+	FUNC_FLAGS_TYPE exec_flags);
+void execute_function_override_wcontext(
+	fvwm_cond_func_rc *cond_rc, const exec_context_t *exc, char *action,
+	FUNC_FLAGS_TYPE exec_flags, int wcontext);
+void execute_function_override_window(
+	fvwm_cond_func_rc *cond_rc, const exec_context_t *exc, char *action,
+	FUNC_FLAGS_TYPE exec_flags, FvwmWindow *fw);
 
 #endif /* FUNCTIONS_H */
