@@ -53,8 +53,8 @@
 #include <X11/Xutil.h>
 #include <X11/cursorfont.h>
 #include <X11/Intrinsic.h>
-#include <libs/PictureBase.h>
-#include <libs/Flocale.h>
+#include "libs/PictureBase.h"
+#include "libs/Flocale.h"
 #include "window_flags.h"
 
 /* ---------------------------- global definitions -------------------------- */
@@ -85,12 +85,9 @@
  */
 
 /* Macro for args passed to fvwm commands... */
-#define F_CMD_ARGS fvwm_cond_func_rc *cond_rc, XEvent *eventp, Window w, \
-	FvwmWindow *fw, unsigned long context,char *action, int *Module
-#define F_PASS_ARGS cond_rc, eventp, w, fw, context, action, Module
-#define F_EXEC_ARGS fvwm_cond_func_rc *cond_rc, char *action, \
-	FvwmWindow *fw, XEvent *eventp, unsigned long context, int Module
-#define F_PASS_EXEC_ARGS cond_rc, action, fw, eventp, context, *Module
+#define F_CMD_ARGS \
+	fvwm_cond_func_rc *cond_rc, const exec_context_t *exc, char *action
+#define F_PASS_ARGS cond_rc, exc, action
 #define FUNC_FLAGS_TYPE unsigned char
 
 /* access macros */
@@ -108,6 +105,8 @@
 
 /* ---------------------------- forward declarations ------------------------ */
 
+struct exec_context_t;
+
 #ifdef USEDECOR
 /* definition in screen.h */
 struct FvwmDecor;
@@ -184,32 +183,6 @@ typedef enum
 	COND_RC_OK = 1
 } fvwm_cond_func_rc;
 
-typedef struct
-{
-	/* return code for conditional commands only */
-	fvwm_cond_func_rc *cond_rc;
-	/* pointer to the event that caused the function */
-	XEvent *eventp;
-	/* the fvwm window structure */
-	struct FvwmWindow *fw;
-	/* the action to execute */
-	char *action;
-	char **args;
-	/* the context in which the button was pressed */
-	unsigned long context;
-	int module;
-	/* If fw is NULL, the is_window_unmanaged flag may be set along
-	 * with this field to indicate the function should run with an
-	 * unmanaged window. */
-	Window win;
-	struct
-	{
-		FUNC_FLAGS_TYPE exec;
-		unsigned do_save_tmpwin : 1;
-		unsigned is_window_unmanaged : 1;
-	} flags;
-} exec_func_args_type;
-
 /*
   For 1 style statement, there can be any number of IconBoxes.
   The name list points at the first one in the chain.
@@ -413,6 +386,7 @@ typedef struct
 #define ICON_HINT_MULTIPLE 2
 	unsigned was_icon_hint_provided : 2;
 	unsigned was_icon_name_provided : 1;
+	unsigned was_never_drawn : 1;
 	unsigned has_ewmh_wm_name : 1;
 	unsigned has_ewmh_wm_icon_name : 1;
 #define EWMH_NO_ICON     0 /* the application does not provide an ewmh icon */
@@ -816,8 +790,12 @@ typedef struct FvwmWindow
 	/* memory for the initial _NET_WM_STATE */
 	unsigned long ewmh_hint_desktop;
 
-	/* multi purpose scratch pointer */
-	void *pscratch;
+	/* multi purpose scratch structure */
+	struct
+	{
+		void *p;
+		int i;
+	} scratch;
 } FvwmWindow;
 
 /* ---------------------------- exported variables (globals) ---------------- */
