@@ -21,12 +21,9 @@
 #include <stdio.h>
 
 #include "libs/fvwmlib.h"
-#include <libs/gravity.h>
 #include "fvwm.h"
 #include "externs.h"
-#include "cursor.h"
-#include "functions.h"
-#include "bindings.h"
+#include "execcontext.h"
 #include "misc.h"
 #include "screen.h"
 #include "geometry.h"
@@ -445,7 +442,7 @@ void get_shaded_client_window_pos(
 
 /* returns the dimensions of the borders */
 void get_window_borders(
-	FvwmWindow *fw, size_borders *borders)
+	const FvwmWindow *fw, size_borders *borders)
 {
 	borders->top_left.width = fw->boundary_width;
 	borders->bottom_right.width = fw->boundary_width;
@@ -476,7 +473,7 @@ void get_window_borders(
 
 /* returns the dimensions of the borders without the title */
 void get_window_borders_no_title(
-	FvwmWindow *fw, size_borders *borders)
+	const FvwmWindow *fw, size_borders *borders)
 {
 	borders->top_left.width = fw->boundary_width;
 	borders->bottom_right.width = fw->boundary_width;
@@ -651,8 +648,8 @@ void maximize_adjust_offset(FvwmWindow *fw)
 ***********************************************************************/
 #define MAKEMULT(a,b) ((b==1) ? (a) : (((int)((a)/(b))) * (b)) )
 void constrain_size(
-	FvwmWindow *fw, unsigned int *widthp, unsigned int *heightp,
-	int xmotion, int ymotion, int flags)
+	FvwmWindow *fw, const XEvent *e, unsigned int *widthp,
+	unsigned int *heightp, int xmotion, int ymotion, int flags)
 {
 	int minWidth, minHeight, maxWidth, maxHeight, xinc, yinc, delta;
 	int baseWidth, baseHeight;
@@ -744,23 +741,23 @@ void constrain_size(
 	 * Step 2a: check we didn't move the edge off screen in interactive
 	 * moves
 	 */
-	if ((flags & CS_ROUND_UP) && Event.type == MotionNotify)
+	if ((flags & CS_ROUND_UP) && e != NULL && e->type == MotionNotify)
 	{
-		if (xmotion > 0 && Event.xmotion.x_root < roundUpX)
+		if (xmotion > 0 && e->xmotion.x_root < roundUpX)
 		{
 			dwidth -= xinc;
 		}
 		else if (xmotion < 0 &&
-			 Event.xmotion.x_root >= Scr.MyDisplayWidth - roundUpX)
+			 e->xmotion.x_root >= Scr.MyDisplayWidth - roundUpX)
 		{
 			dwidth -= xinc;
 		}
-		if (ymotion > 0 && Event.xmotion.y_root < roundUpY)
+		if (ymotion > 0 && e->xmotion.y_root < roundUpY)
 		{
 			dheight -= yinc;
 		}
 		else if (ymotion < 0 &&
-			 Event.xmotion.y_root >= Scr.MyDisplayHeight - roundUpY)
+			 e->xmotion.y_root >= Scr.MyDisplayHeight - roundUpY)
 		{
 			dheight -= yinc;
 		}
@@ -935,8 +932,8 @@ void gravity_constrain_size(
 		new_height = rect->height;
 	}
 	constrain_size(
-		t, (unsigned int *)&new_width, (unsigned int *)&new_height, 0,
-		0, flags);
+		t, NULL, (unsigned int *)&new_width,
+		(unsigned int *)&new_height, 0, 0, flags);
 	if (rect->width != new_width || rect->height != new_height)
 	{
 		gravity_resize(
@@ -1079,10 +1076,25 @@ void move_icon_to_position(
 {
 	if (fw->icon_g.picture_w_g.width > 0)
 	{
+		int cs;
+
+		if (Scr.Hilite == fw)
+		{
+			cs = fw->cs_hi;
+		}
+		else
+		{
+			cs = fw->cs;
+		}
 		XMoveWindow(
 			dpy, FW_W_ICON_PIXMAP(fw),
 			fw->icon_g.picture_w_g.x,
 			fw->icon_g.picture_w_g.y);
+		if (fw->icon_alphaPixmap ||
+		    (cs >= 0 && Colorset[cs].icon_alpha_percent < 100))
+		{
+			DrawIconWindow(fw, False, True, False, NULL);
+		}
 	}
 	if (!HAS_NO_ICON_TITLE(fw))
 	{
@@ -1090,10 +1102,6 @@ void move_icon_to_position(
 			dpy, FW_W_ICON_TITLE(fw),
 			fw->icon_g.title_w_g.x,
 			fw->icon_g.title_w_g.y);
-		if (fw->icon_alphaPixmap)
-		{
-			DrawIconWindow(fw);
-		}
 	}
 
 	return;
