@@ -20,7 +20,7 @@
 void GNOME_Init(void);
 
 /* client messages; setting hints on a window comes through this mechanism */
-int  GNOME_ProcessClientMessage(FvwmWindow *fwin, XEvent *ev);
+int  GNOME_ProcessClientMessage(const exec_context_t *exc);
 
 /* hook into .fvwm2rc functions */
 void GNOME_ButtonFunc(
@@ -31,7 +31,7 @@ void GNOME_ButtonFunc(
 	 char *action,
 	 int *Module);
 
-void GNOME_ProxyButtonEvent(XEvent *ev);
+void GNOME_ProxyButtonEvent(const XEvent *ev);
 
 void GNOME_ShowDesks(
 	 XEvent *eventp,
@@ -52,10 +52,8 @@ void GNOME_SetHints(FvwmWindow *fwin);
 void GNOME_SetLayer(FvwmWindow *fwin);
 void GNOME_SetDesk(FvwmWindow *fwin);
 void GNOME_SetWinArea(FvwmWindow *w);
-void GNOME_HandlePropRequest(unsigned int propm,
-			     unsigned int prop,
-			     Window win,
-			     XEvent *ev);
+void GNOME_HandlePropRequest(
+	const exec_context_t *exc, unsigned int propm, unsigned int prop);
 
 /* update public window manager information */
 void GNOME_SetAreaCount(void);
@@ -67,7 +65,7 @@ void GNOME_SetClientList(void);
 #else
 
 #define GNOME_Init()
-#define GNOME_ProcessClientMessage(fwin, ev) 0
+#define GNOME_ProcessClientMessage(exc) 0
 #define GNOME_ButtonFunc(eventp, w, fwin, context, action, Module)
 #define GNOME_ProxyButtonEvent(ev)
 #define GNOME_ShowDesks(eventp, w, fwin, context, action, Module)
@@ -77,7 +75,7 @@ void GNOME_SetClientList(void);
 #define GNOME_SetLayer(fwin)
 #define GNOME_SetDesk(fwin)
 #define GNOME_SetWinArea(w)
-#define GNOME_HandlePropRequest(propm, prop, win, ev)
+#define GNOME_HandlePropRequest(exc, propm, prop)
 #define GNOME_SetAreaCount()
 #define GNOME_SetDeskCount()
 #define GNOME_SetCurrentArea()
