@@ -28,19 +28,20 @@
 
 #include <stdio.h>
 #include <stdarg.h>
-#include "libs/ftime.h"
 
-#include "libs/fvwmlib.h"
+#include "libs/ftime.h"
+#ifdef FVWM_DEBUG_TIME
+#include <sys/times.h>
+#endif
 #include "fvwm.h"
 #include "externs.h"
+#include "execcontext.h"
 #include "cursor.h"
-#include "functions.h"
-#include "bindings.h"
 #include "misc.h"
 #include "screen.h"
 #include "module_interface.h"
 #include "events.h"
-#include "focus.h"
+#include "eventmask.h"
 
 /* ---------------------------- local definitions --------------------------- */
 
@@ -335,19 +336,22 @@ Bool UngrabEm(int ungrab_context)
 ** id -> name of function, or other identifier
 */
 static char *fvwm_msg_strings[] =
-{ "<<DEBUG>>", "", "", "<<WARNING>>", "<<DEPRECATED>>", "<<ERROR>>" };
+{
+	"<<DEBUG>> ", "", "", "<<WARNING>> ", "<<DEPRECATED>> ", "<<ERROR>> "
+};
+
 void fvwm_msg(fvwm_msg_type type, char *id, char *msg, ...)
 {
 	va_list args;
+	char fvwm_id[20];
+	char time_str[40] = "\0";
 #ifdef FVWM_DEBUG_TIME
 	clock_t time_val, time_taken;
 	static clock_t start_time = 0;
 	static clock_t prev_time = 0;
 	struct tms not_used_tms;
-	char buffer[200];                     /* oversized */
 	time_t mytime;
 	struct tm *t_ptr;
-	static int counter = 0;
 #endif
 
 #ifdef FVWM_DEBUG_TIME
@@ -361,35 +365,19 @@ void fvwm_msg(fvwm_msg_type type, char *id, char *msg, ...)
 	time_val = (unsigned int)times(&not_used_tms); /* get clock ticks */
 	time_taken = time_val - prev_time;
 	prev_time = time_val;
-	sprintf(buffer, "%.2d:%.2d:%.2d %6ld",
+	sprintf(time_str, "%.2d:%.2d:%.2d%7ld ",
 		t_ptr->tm_hour, t_ptr->tm_min, t_ptr->tm_sec, time_taken);
 #endif
 
+	strcpy(fvwm_id, "FVWM");
 	if (Scr.NumberOfScreens > 1)
 	{
-		fprintf(stderr,"[FVWM.%d][%s]: "
-#ifdef FVWM_DEBUG_TIME
-			"%s "
-#endif
-			"%s ",(int)Scr.screen,id,
-#ifdef FVWM_DEBUG_TIME
-			buffer,
-#endif
-			fvwm_msg_strings[(int)type]);
-	}
-	else
-	{
-		fprintf(stderr,"[FVWM][%s]: "
-#ifdef FVWM_DEBUG_TIME
-			"%s "
-#endif
-			"%s ",id,
-#ifdef FVWM_DEBUG_TIME
-			buffer,
-#endif
-			fvwm_msg_strings[(int)type]);
+		sprintf(&fvwm_id[strlen(fvwm_id)], ".%d", (int)Scr.screen);
 	}
 
+	fprintf(stderr, "%s[%s][%s]: %s",
+		time_str, fvwm_id, id, fvwm_msg_strings[(int)type]);
+
 	if (type == ECHO)
 	{
 		/* user echos must be printed as a literal string */
@@ -397,27 +385,28 @@ void fvwm_msg(fvwm_msg_type type, char *id, char *msg, ...)
 	}
 	else
 	{
-		va_start(args,msg);
+		va_start(args, msg);
 		vfprintf(stderr, msg, args);
 		va_end(args);
 	}
-	fprintf(stderr,"\n");
+	fprintf(stderr, "\n");
 
 	if (type == ERR)
 	{
 		/* I hate to use a fixed length but this will do for now */
 		char tmp[2 * MAX_TOKEN_LENGTH];
-		sprintf(tmp,"[FVWM][%s]: %s ",id, fvwm_msg_strings[(int)type]);
-		va_start(args,msg);
-		vsprintf(tmp+strlen(tmp), msg, args);
+		sprintf(tmp, "[%s][%s]: %s",
+			fvwm_id, id, fvwm_msg_strings[(int)type]);
+		va_start(args, msg);
+		vsprintf(tmp + strlen(tmp), msg, args);
 		va_end(args);
-		tmp[strlen(tmp)+1]='\0';
-		tmp[strlen(tmp)]='\n';
+		tmp[strlen(tmp) + 1] = '\0';
+		tmp[strlen(tmp)] = '\n';
 		if (strlen(tmp) >= MAX_MODULE_INPUT_TEXT_LEN)
 		{
 			sprintf(tmp + MAX_MODULE_INPUT_TEXT_LEN - 5, "...\n");
 		}
-		BroadcastName(M_ERROR,0,0,0,tmp);
+		BroadcastName(M_ERROR, 0, 0, 0, tmp);
 	}
 
 } /* fvwm_msg */
@@ -457,7 +446,7 @@ void NewFontAndColor(FlocaleFont *flf, Pixel color, Pixel backcolor)
  *
  ****************************************************************************/
 void Keyboard_shortcuts(
-	XEvent *Event, FvwmWindow *fw, int *x_defect, int *y_defect,
+	XEvent *ev, FvwmWindow *fw, int *x_defect, int *y_defect,
 	int ReturnEvent)
 {
 	int x_move_size = 0;
@@ -469,7 +458,7 @@ void Keyboard_shortcuts(
 		y_move_size = fw->hints.height_inc;
 	}
 	fvwmlib_keyboard_shortcuts(
-		dpy, Scr.screen, Event, x_move_size, y_move_size, x_defect,
+		dpy, Scr.screen, ev, x_move_size, y_move_size, x_defect,
 		y_defect, ReturnEvent);
 
 	return;
@@ -500,7 +489,7 @@ int truncate_to_multiple (int x, int m)
 	return (x < 0) ? (m * (((x + 1) / m) - 1)) : (m * (x / m));
 }
 
-Bool IsRectangleOnThisPage(rectangle *rec, int desk)
+Bool IsRectangleOnThisPage(const rectangle *rec, int desk)
 {
 	return (desk == Scr.CurrentDesk &&
 		rec->x + rec->width > 0 &&
@@ -518,7 +507,7 @@ FvwmWindow *get_pointer_fvwm_window(void)
 	Window ancestor;
 	FvwmWindow *t;
 
-	if (XQueryPointer(
+	if (FQueryPointer(
 		    dpy, Scr.Root, &JunkRoot, &win, &JunkX, &JunkY,
 		    &x, &y, &JunkMask) == False)
 	{
@@ -567,10 +556,9 @@ Time get_server_time(void)
 	XChangeProperty(
 		dpy, Scr.NoFocusWin, XA_WM_CLASS, XA_STRING, 8, PropModeAppend,
 		NULL, 0);
-	XWindowEvent(dpy, Scr.NoFocusWin, PropertyChangeMask, &xev);
+	FWindowEvent(dpy, Scr.NoFocusWin, PropertyChangeMask, &xev);
 	attr.event_mask = XEVMASK_NOFOCUSW;
 	XChangeWindowAttributes(dpy, Scr.NoFocusWin, CWEventMask, &attr);
-	StashEventTime(&xev);
 
 	return xev.xproperty.time;
 }
