@@ -8,12 +8,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 /* FlocaleRotateDrawString is strongly inspired by some part of xvertext
@@ -27,7 +27,7 @@
  * copyright notice and this permission notice appear in supporting
  * documentation.  All work developed as a consequence of the use of
  * this program should duly acknowledge such use. No representations are
- * made about the suitability of this software for any purpose.	 It is
+ * made about the suitability of this software for any purpose.  It is
  * provided "as is" without express or implied warranty.
  *
  * Minor modifications by Chris Cannam for wm2/wmx
@@ -64,6 +64,15 @@
 
 /* ---------------------------- local macros -------------------------------- */
 
+#define FSwitchDrawString(use_16, dpy, d, gc, x, y, s8, s16, l) \
+	(use_16) ? \
+	XDrawString16(dpy, d, gc, x, y, s16, l) : \
+	XDrawString(dpy, d, gc, x, y, s8, l)
+#define FSwitchDrawImageString(use_16, dpy, d, gc, x, y, s8, s16, l) \
+	(use_16) ? \
+	XDrawImageString16(dpy, d, gc, x, y, s16, l) : \
+	XDrawImageString(dpy, d, gc, x, y, s8, l)
+
 /* ---------------------------- imports ------------------------------------- */
 
 /* ---------------------------- included code files ------------------------- */
@@ -77,7 +86,6 @@
 static FlocaleFont *FlocaleFontList = NULL;
 static char *Flocale = NULL;
 static char *Fmodifiers = NULL;
-static Bool FlocaleSeted = False;
 
 /* ---------------------------- exported variables (globals) ---------------- */
 
@@ -124,9 +132,9 @@ void FlocaleParseShadow(char *str, int *shadow_size, int *shadow_offset,
 		}
 		PeekToken(dir_str, &dir_str);
 	}
-	while(dir_str && *dir_str && *dir_str != '\n')
+	while (dir_str && *dir_str && *dir_str != '\n')
 	{
-		dir = ParseMultiDirectionArgument(dir_str, &dir_str);
+		dir = gravity_parse_multi_dir_argument(dir_str, &dir_str);
 		if (dir == MULTI_DIR_NONE)
 		{
 			fprintf(stderr,"[%s][FlocaleParseShadow]: WARNING -- "
@@ -155,7 +163,7 @@ XChar2b *FlocaleUtf8ToUnicodeStr2b(unsigned char *str, int len, int *nl)
 	int i = 0, j = 0, t;
 
 	str2b = (XChar2b *)safemalloc((len+1)*sizeof(XChar2b));
-	while(i < len && str[i] != 0)
+	while (i < len && str[i] != 0)
 	{
 		if (str[i] <= 0x7f)
 		{
@@ -219,7 +227,7 @@ XChar2b *FlocaleStringToString2b(
 	str2b = (XChar2b *)safemalloc((len+1)*sizeof(XChar2b));
 	if (euc)
 	{
-		while(i < len && str[i] != 0)
+		while (i < len && str[i] != 0)
 		{
 			if (str[i] <= 0x7f)
 			{
@@ -246,19 +254,19 @@ XChar2b *FlocaleStringToString2b(
 	}
 	else /* big5 and others not yet tested */
 	{
-		while(i < len && str[i] != 0)
+		while (i < len && str[i] != 0)
 		{
 			if (str[i] <= 0x7f)
 			{
 				/* we should convert to ascii */
-#if 0 
+#if 0
 				str2b[j].byte1 = 0xa2; /* magic number! */
 				str2b[j].byte2 = str[i++];
 #endif
 				/* a blanck char ... */
 				str2b[j].byte1 = 0x21;
 				str2b[j].byte2 = 0x21;
-			} 
+			}
 			else if (i+1 < len)
 			{
 				str2b[j].byte1 = str[i++];
@@ -271,7 +279,7 @@ XChar2b *FlocaleStringToString2b(
 				i++;
 			}
 			j++;
-		}	
+		}
 	}
 	*nl = j;
 	if (free_str)
@@ -345,7 +353,7 @@ void FlocaleEncodeWinString(
 	fws->e_str = FlocaleEncodeString(
 		dpy, flf, fws->str, do_free, *len, len, NULL);
 	fws->str2b = NULL;
-	
+
 	if (flf->font != None)
 	{
 		if (FLC_ENCODING_TYPE_IS_UTF_8(flf->fc))
@@ -365,7 +373,7 @@ void FlocaleEncodeWinString(
 				free(fws->e_str);
 			}
 			fws->e_str = NULL;
-			*do_free = True;	
+			*do_free = True;
 		}
 	}
 }
@@ -380,65 +388,47 @@ void FlocaleFontStructDrawString(Display *dpy, FlocaleFont *flf, Drawable d,
 				 Bool has_fg_pixels, FlocaleWinString *fws,
 				 int len, Bool image)
 {
-	int xt = x, yt = y, step = 0;
+	int xt = x;
+	int yt = y;
+	int is_string16;
+	flocale_gstp_args gstp_args;
 
-	if (FLC_ENCODING_TYPE_IS_UTF_8(flf->fc) || flf->flags.is_mb)
+	is_string16 = (FLC_ENCODING_TYPE_IS_UTF_8(flf->fc) || flf->flags.is_mb);
+	if (is_string16 && fws->str2b == NULL)
 	{
-		if (fws->str2b != NULL)
-		{
-			if (image)
-			{
-				/* for rotated drawing */
-				XDrawImageString16(
-					dpy, d, gc, x, y, fws->str2b, len);
-			}
-			else
-			{
-				/* normal drawing */
-				if (flf->shadow_size != 0 && has_fg_pixels)
-				{
-					XSetForeground(dpy, fws->gc, fgsh);
-					while(FlocaleGetShadowTextPosition(
-						flf, fws, x, y, &xt, &yt, &step))
-					{
-						XDrawString16(
-							dpy, d, gc, xt, yt,
-							fws->str2b, len);
-					}
-					XSetForeground(dpy, gc, fg);
-				}
-				XDrawString16(
-					dpy, d, gc, xt,yt, fws->str2b, len);
-			}
-		}
+		return;
+	}
+	if (image)
+	{
+		/* for rotated drawing */
+		FSwitchDrawImageString(
+			is_string16, dpy, d, gc, x, y, fws->e_str, fws->str2b,
+			len);
 	}
 	else
 	{
-		if (image)
+		FlocaleInitGstpArgs(&gstp_args, flf, fws, x, y);
+		/* normal drawing */
+		if (flf->shadow_size != 0 && has_fg_pixels)
 		{
-			/* for rotated drawing */
-			XDrawImageString(dpy, d, gc, x, y, fws->e_str, len);
-		}
-		else
-		{
-			/* normale drawing */
-			if (flf->shadow_size != 0 && has_fg_pixels)
+			XSetForeground(dpy, fws->gc, fgsh);
+			while (FlocaleGetShadowTextPosition(
+				       &xt, &yt, &gstp_args))
 			{
-				XSetForeground(dpy, fws->gc, fgsh);
-				while(FlocaleGetShadowTextPosition(
-						     flf, fws, x, y,
-						     &xt, &yt, &step))
-				{
-					XDrawString(
-						dpy, d, gc,
-						xt, yt,
-						fws->e_str, len);
-				}
-				XSetForeground(dpy, gc, fg);
+				FSwitchDrawString(
+					is_string16, dpy, d, gc, xt, yt,
+					fws->e_str, fws->str2b, len);
 			}
-			XDrawString(dpy, d, gc, xt, yt, fws->e_str, len);
+			XSetForeground(dpy, gc, fg);
 		}
+		xt = gstp_args.orig_x;
+		yt = gstp_args.orig_y;
+		FSwitchDrawString(
+			is_string16, dpy, d, gc, xt,yt, fws->e_str, fws->str2b,
+			len);
 	}
+
+	return;
 }
 
 /* ***************************************************************************
@@ -449,9 +439,9 @@ void FlocaleRotateDrawString(
 	Display *dpy, FlocaleFont *flf, FlocaleWinString *fws, Pixel fg,
 	Pixel fgsh, Bool has_fg_pixels, int len)
 {
-	static GC my_gc, font_gc;
+	static GC my_gc = None;
+	static GC font_gc = None;
 	int j, i, xpfg, ypfg, xpsh, ypsh;
-	int step = 0;
 	unsigned char *normal_data, *rotated_data;
 	unsigned int normal_w, normal_h, normal_len;
 	unsigned int rotated_w, rotated_h, rotated_len;
@@ -459,13 +449,17 @@ void FlocaleRotateDrawString(
 	int width, height, descent;
 	XImage *image, *rotated_image;
 	Pixmap canvas_pix, rotated_pix;
+	flocale_gstp_args gstp_args;
 
 	if (fws->str == NULL || len < 1)
 		return;
-	if (fws->flags.text_rotation == TEXT_ROTATED_0)
+	if (fws->flags.text_rotation == ROTATION_0)
 		return; /* should not happen */
 
-	my_gc = fvwmlib_XCreateGC(dpy, fws->win, 0, NULL);
+	if (my_gc == None)
+	{
+		my_gc = fvwmlib_XCreateGC(dpy, fws->win, 0, NULL);
+	}
 	XCopyGC(dpy, fws->gc, GCForeground|GCBackground, my_gc);
 
 	/* width and height (no shadow!) */
@@ -473,8 +467,10 @@ void FlocaleRotateDrawString(
 	height = flf->height - FLF_SHADOW_HEIGHT(flf);
 	descent = flf->descent - FLF_SHADOW_DESCENT(flf);;
 
-	if (width < 1) width = 1;
-	if (height < 1) height = 1;
+	if (width < 1)
+		width = 1;
+	if (height < 1)
+		height = 1;
 
 	/* glyph width and height of the normal text */
 	normal_w = width;
@@ -485,7 +481,10 @@ void FlocaleRotateDrawString(
 
 	/* create and clear the canvas */
 	canvas_pix = XCreatePixmap(dpy, fws->win, width, height, 1);
-	font_gc = fvwmlib_XCreateGC(dpy, canvas_pix, 0, NULL);
+	if (font_gc == None)
+	{
+		font_gc = fvwmlib_XCreateGC(dpy, canvas_pix, 0, NULL);
+	}
 	XSetBackground(dpy, font_gc, 0);
 	XSetForeground(dpy, font_gc, 0);
 	XFillRectangle(dpy, canvas_pix, font_gc, 0, 0, width, height);
@@ -525,7 +524,7 @@ void FlocaleRotateDrawString(
 	image->format = XYBitmap;
 
 	/* width, height of the rotated text */
-	if (fws->flags.text_rotation == TEXT_ROTATED_180)
+	if (fws->flags.text_rotation == ROTATION_180)
 	{
 		rotated_w = normal_w;
 		rotated_h = normal_h;
@@ -558,19 +557,19 @@ void FlocaleRotateDrawString(
 		for (i = 0; i < rotated_w; i++)
 		{
 			/* map bits ... */
-			if (fws->flags.text_rotation == TEXT_ROTATED_270)
+			if (fws->flags.text_rotation == ROTATION_270)
 				val = normal_data[
 					i * normal_len +
 					(normal_w - j - 1) / 8
 				] & (128 >> ((normal_w - j - 1) % 8));
 
-			else if (fws->flags.text_rotation == TEXT_ROTATED_180)
+			else if (fws->flags.text_rotation == ROTATION_180)
 				val = normal_data[
 					(normal_h - j - 1) * normal_len +
 					(normal_w - i - 1) / 8
 				] & (128 >> ((normal_w - i - 1) % 8));
 
-			else /* TEXT_ROTATED_90 */
+			else /* ROTATION_90 */
 				val = normal_data[
 					(normal_h - i - 1) * normal_len +
 					j / 8
@@ -594,52 +593,56 @@ void FlocaleRotateDrawString(
 
 	/* free pixmap and GC */
 	XFreePixmap(dpy, canvas_pix);
-	XFreeGC(dpy, font_gc);
 
 	/* x and y corrections: we fill a rectangle! */
-	if (fws->flags.text_rotation == TEXT_ROTATED_270) /* CCW */
+	switch (fws->flags.text_rotation)
 	{
-		xpfg = fws->x - flf->ascent;
+	case ROTATION_90:
+		/* CW */
+		xpfg = fws->x - (flf->height - flf->ascent);
 		ypfg = fws->y;
-	}
-	else if (fws->flags.text_rotation == TEXT_ROTATED_180)
-	{
+		break;
+	case ROTATION_180:
 		xpfg = fws->x;
 		ypfg = fws->y - (flf->height - flf->ascent) +
 			FLF_SHADOW_BOTTOM_SIZE(flf);
-	}
-	else /* fws->flags.text_rotation == TEXT_ROTATED_90 (CW) */
-	{
-		xpfg = fws->x - (flf->height - flf->ascent);
+		break;
+	case ROTATION_270:
+		/* CCW */
+		xpfg = fws->x - flf->ascent;
 		ypfg = fws->y;
+		break;
+	case ROTATION_0:
+	default:
+		xpfg = fws->x;
+		ypfg = fws->y;
+		break;
 	}
 	xpsh = xpfg;
-	ypsh = ypsh;
+	ypsh = ypfg;
 	/* write the image on the window */
 	XSetFillStyle(dpy, my_gc, FillStippled);
 	XSetStipple(dpy, my_gc, rotated_pix);
+	FlocaleInitGstpArgs(&gstp_args, flf, fws, xpfg, ypfg);
 	if (flf->shadow_size != 0 && has_fg_pixels)
 	{
 		XSetForeground(dpy, my_gc, fgsh);
-		while(FlocaleGetShadowTextPosition(flf, fws, xpfg, ypfg,
-						   &xpsh, &ypsh, &step))
+		while (FlocaleGetShadowTextPosition(&xpsh, &ypsh, &gstp_args))
 		{
 			XSetTSOrigin(dpy, my_gc, xpsh, ypsh);
-			XFillRectangle(dpy, fws->win, my_gc, xpsh, ypsh,
-				       rotated_w, rotated_h);
+			XFillRectangle(
+				dpy, fws->win, my_gc, xpsh, ypsh, rotated_w,
+				rotated_h);
 		}
 		XSetForeground(dpy, my_gc, fg);
 	}
-	else
-	{
-		FlocaleGetShadowTextPosition(flf, fws, xpfg, ypfg,
-					     &xpsh, &ypsh, &step);
-	}
+	xpsh = gstp_args.orig_x;
+	ypsh = gstp_args.orig_y;
 	XSetTSOrigin(dpy, my_gc, xpsh, ypsh);
 	XFillRectangle(dpy, fws->win, my_gc, xpsh, ypsh, rotated_w, rotated_h);
-
 	XFreePixmap(dpy, rotated_pix);
-	XFreeGC(dpy, my_gc);
+
+	return;
 }
 
 /* ***************************************************************************
@@ -852,8 +855,6 @@ static
 void FlocaleSetlocaleForX(
 	int category, const char *locale, const char *module)
 {
-	FlocaleSeted = True;
-
 	if ((Flocale = setlocale(category, locale)) == NULL)
 	{
 		fprintf(stderr,
@@ -959,7 +960,7 @@ FlocaleFont *FlocaleLoadFont(Display *dpy, char *fontname, char *module)
 
 	/* But first see if we have a shadow relief and/or an encoding */
 	str = fontname;
-	while ((i = GetTokenIndex(str, prefix_list, -1, &str)) > -1) 
+	while ((i = GetTokenIndex(str, prefix_list, -1, &str)) > -1)
 	{
 		str = GetQuotedString(str, &opt_str, ":", NULL, NULL, NULL);
 		switch(i)
@@ -1052,7 +1053,8 @@ FlocaleFont *FlocaleLoadFont(Display *dpy, char *fontname, char *module)
 			}
 			if ((flf =
 			     FlocaleGetFont(
-			       dpy, FLOCALE_FALLBACK_FONT, NULL, module))!= NULL)
+			       dpy, FLOCALE_FALLBACK_FONT, NULL, module)) !=
+			    NULL)
 			{
 				flf->name = FLOCALE_FALLBACK_FONT;
 			}
@@ -1118,6 +1120,7 @@ FlocaleFont *FlocaleLoadFont(Display *dpy, char *fontname, char *module)
 	{
 		free(encoding);
 	}
+
 	return flf;
 }
 
@@ -1168,7 +1171,7 @@ void FlocaleUnloadFont(Display *dpy, FlocaleFont *flf)
 			free(flf->fc->bidi);
 		if (flf->fc->locale != NULL)
 		{
-			while(FLC_GET_LOCALE_CHARSET(flf->fc,i) != NULL)
+			while (FLC_GET_LOCALE_CHARSET(flf->fc,i) != NULL)
 			{
 				free(FLC_GET_LOCALE_CHARSET(flf->fc,i));
 				i++;
@@ -1204,295 +1207,136 @@ void FlocaleUnloadFont(Display *dpy, FlocaleFont *flf)
  * Width and Drawing Text
  * ***************************************************************************/
 
-Bool FlocaleGetShadowTextPosition(FlocaleFont *flf, FlocaleWinString *fws,
-				  int orig_x, int orig_y,
-				  int *x, int *y, int *step)
+void FlocaleInitGstpArgs(
+	flocale_gstp_args *args, FlocaleFont *flf, FlocaleWinString *fws,
+	int start_x, int start_y)
 {
-
-	static multi_direction_type direction = MULTI_DIR_NONE;
-	static unsigned short inter_step = 0;
-	static unsigned short x_sign = 0, y_sign = 0;
-
-	if (*step == 0)
+	args->step = 0;
+	args->offset = flf->shadow_offset + 1;
+	args->outer_offset = flf->shadow_offset + flf->shadow_size;
+	args->size = flf->shadow_size;
+	args->sdir = flf->flags.shadow_dir;
+	switch (fws->flags.text_rotation)
 	{
-		direction = MULTI_DIR_NONE;
+	case ROTATION_270: /* CCW */
+		args->orig_x = start_x + FLF_SHADOW_UPPER_SIZE(flf);
+		args->orig_y = start_y + FLF_SHADOW_RIGHT_SIZE(flf);
+		break;
+	case ROTATION_180:
+		args->orig_x = start_x + FLF_SHADOW_RIGHT_SIZE(flf);
+		args->orig_y = start_y;
+		break;
+	case ROTATION_90: /* CW */
+		args->orig_x = start_x + FLF_SHADOW_BOTTOM_SIZE(flf);
+		args->orig_y = start_y + FLF_SHADOW_LEFT_SIZE(flf);
+		break;
+	case ROTATION_0:
+	default:
+		args->orig_x = start_x + FLF_SHADOW_LEFT_SIZE(flf);
+		args->orig_y = start_y;
+		break;
 	}
-	if ((*step == 0 || inter_step == flf->shadow_size) &&
-	    flf->shadow_size != 0)
+	args->rot = fws->flags.text_rotation;
+
+	return;
+}
+
+Bool FlocaleGetShadowTextPosition(
+	int *x, int *y, flocale_gstp_args *args)
+{
+	if (args->step == 0)
+	{
+		args->direction = MULTI_DIR_NONE;
+		args->inter_step = 0;
+	}
+	if ((args->step == 0 || args->inter_step >= args->num_inter_steps) &&
+	    args->size != 0)
 	{
 		/* setup a new direction */
-		inter_step = 0;
-		GetNextMultiDirection(flf->flags.shadow_dir, &direction);
-	}
-	if (direction == MULTI_DIR_NONE || flf->shadow_size == 0)
-	{
-		/* finished; return the position for the no shadow drawing */
-		switch(fws->flags.text_rotation)
+		args->inter_step = 0;
+		gravity_get_next_multi_dir(args->sdir, &args->direction);
+		if (args->direction == MULTI_DIR_C)
 		{
-		case TEXT_ROTATED_270: /* CCW */
-#define TR_CCW_ORIG_X  orig_x + FLF_SHADOW_UPPER_SIZE(flf)
-#define TR_CCW_ORIG_Y  orig_y + FLF_SHADOW_RIGHT_SIZE(flf)
-			*x = TR_CCW_ORIG_X;
-			*y = TR_CCW_ORIG_Y;
-			break;
-		case TEXT_ROTATED_180:
-#define REVERSE_ORIG_X	orig_x + FLF_SHADOW_RIGHT_SIZE(flf)
-#define REVERSE_ORIG_Y	orig_y
-			*x = REVERSE_ORIG_X;
-			*y = REVERSE_ORIG_Y;
-			break;
-		case TEXT_ROTATED_90: /* CW */
-#define TR_CW_ORIG_X  orig_x + FLF_SHADOW_BOTTOM_SIZE(flf)
-#define TR_CW_ORIG_Y  orig_y + FLF_SHADOW_LEFT_SIZE(flf)
-			*x = TR_CW_ORIG_X;
-			*y = TR_CW_ORIG_Y;
-			break;
-		case TEXT_ROTATED_0:
-		default:
-#define NORMAL_ORIG_X  orig_x + FLF_SHADOW_LEFT_SIZE(flf)
-#define NORMAL_ORIG_Y  orig_y
-			*x = NORMAL_ORIG_X;
-			*y = NORMAL_ORIG_Y;
-			break;
+			int size;
+
+			size = 2 * (args->outer_offset) + 1;
+			args->num_inter_steps = size * size;
 		}
+		else
+		{
+			args->num_inter_steps = args->size;
+		}
+	}
+	if (args->direction == MULTI_DIR_NONE || args->size == 0)
+	{
+		*x = args->orig_x;
+		*y = args->orig_y;
+
 		return False;
 	}
-	if (inter_step > 0)
+	if (args->direction == MULTI_DIR_C)
+	{
+		int tx;
+		int ty;
+		int size;
+		int is_finished;
+
+		size = 2 * (args->outer_offset) + 1;
+		tx = args->inter_step % size - args->outer_offset;
+		ty = args->inter_step / size - args->outer_offset;
+		for (is_finished = 0; ty <= args->outer_offset;
+		     ty++, tx = -args->outer_offset)
+		{
+			for (; tx <= args->outer_offset; tx++)
+			{
+				if (tx <= -args->offset || tx >= args->offset ||
+				    ty <= -args->offset || ty >= args->offset)
+				{
+					is_finished = 1;
+					break;
+				}
+			}
+			if (is_finished)
+			{
+				break;
+			}
+		}
+		args->inter_step =
+			(tx + args->outer_offset) +
+			(ty + args->outer_offset) * size;
+		if (!is_finished)
+		{
+			tx = 0;
+			ty = 0;
+		}
+		*x = args->orig_x + tx;
+		*y = args->orig_y + ty;
+	}
+	else if (args->inter_step > 0)
 	{
 		/* into a directional drawing */
-		(*x) += x_sign;
-		(*y) += y_sign;
+		(*x) += args->x_sign;
+		(*y) += args->y_sign;
 	}
 	else
 	{
-		/* start a drawing in a given direction */
-		switch(fws->flags.text_rotation)
-		{
-		case TEXT_ROTATED_270: /* CCW */
-			switch(direction)
-			{
-			case MULTI_DIR_W:
-				*x = TR_CCW_ORIG_X;
-				*y = TR_CCW_ORIG_Y + 1 + flf->shadow_offset;
-				x_sign = 0;
-				y_sign = 1;
-				break;
-			case MULTI_DIR_NW:
-				*x = TR_CCW_ORIG_X - 1 - flf->shadow_offset;
-				*y = TR_CCW_ORIG_Y + 1 + flf->shadow_offset;
-				x_sign = -1;
-				y_sign = 1;
-				break;
-			case MULTI_DIR_N:
-				*x = TR_CCW_ORIG_X - 1 - flf->shadow_offset;
-				*y = TR_CCW_ORIG_Y;
-				x_sign = -1;
-				y_sign = 0;
-				break;
-			case MULTI_DIR_NE:
-				*x = TR_CCW_ORIG_X - 1 - flf->shadow_offset;
-				*y = TR_CCW_ORIG_Y - 1 - flf->shadow_offset;
-				x_sign = -1;
-				y_sign = -1;
-				break;
-			case MULTI_DIR_E:
-				*x = TR_CCW_ORIG_X;
-				*y = TR_CCW_ORIG_Y - 1 - flf->shadow_offset;
-				x_sign = 0;
-				y_sign = -1;
-				break;
-			case MULTI_DIR_SE:
-				*x = TR_CCW_ORIG_X + 1 + flf->shadow_offset;
-				*y = TR_CCW_ORIG_Y - 1 - flf->shadow_offset;
-				x_sign = 1;
-				y_sign = -1;
-				break;
-			case MULTI_DIR_S:
-				*x = TR_CCW_ORIG_X + 1 + flf->shadow_offset;
-				*y = TR_CCW_ORIG_Y;
-				x_sign = 1;
-				y_sign = 0;
-				break;
-			case MULTI_DIR_SW:
-				*x = TR_CCW_ORIG_X + 1 + flf->shadow_offset;
-				*y = TR_CCW_ORIG_Y + 1 + flf->shadow_offset;
-				x_sign = 1;
-				y_sign = 1;
-				break;
-			default: /* never happen */
-				break;
-			}
-			break;
-		case TEXT_ROTATED_180: /* (exact "opposite" of normal dir) */
-			switch(direction)
-			{
-			case MULTI_DIR_W:
-				*x = REVERSE_ORIG_X + 1 + flf->shadow_offset;
-				*y = REVERSE_ORIG_Y;
-				x_sign = 1;
-				y_sign = 0;
-				break;
-			case MULTI_DIR_NW:
-				*x = REVERSE_ORIG_X + 1 + flf->shadow_offset;
-				*y = REVERSE_ORIG_Y + 1 + flf->shadow_offset;
-				x_sign = 1;
-				y_sign = 1;
-				break;
-			case MULTI_DIR_N:
-				*x = REVERSE_ORIG_X;
-				*y = REVERSE_ORIG_Y + 1 + flf->shadow_offset;
-				x_sign = 0;
-				y_sign = 1;
-				break;
-			case MULTI_DIR_NE:
-				*x = REVERSE_ORIG_X - 1 - flf->shadow_offset;
-				*y = REVERSE_ORIG_Y + 1 + flf->shadow_offset;
-				x_sign = -1;
-				y_sign = 1;
-				break;
-			case MULTI_DIR_E:
-				*x = REVERSE_ORIG_X - 1 - flf->shadow_offset;
-				*y = REVERSE_ORIG_Y;
-				x_sign = -1;
-				y_sign = 0;
-				break;
-			case MULTI_DIR_SE:
-				*x = REVERSE_ORIG_X - 1 - flf->shadow_offset;
-				*y = REVERSE_ORIG_Y - 1 - flf->shadow_offset;
-				x_sign = -1;
-				y_sign = -1;
-				break;
-			case MULTI_DIR_S:
-				*x = REVERSE_ORIG_X;
-				*y = REVERSE_ORIG_Y - 1 - flf->shadow_offset;
-				x_sign = 0;
-				y_sign = -1;
-				break;
-			case MULTI_DIR_SW:
-				*x = REVERSE_ORIG_X + 1 + flf->shadow_offset;
-				*y = REVERSE_ORIG_Y - 1 - flf->shadow_offset;
-				x_sign = 1;
-				y_sign = -1;
-				break;
-			default: /* never happen */
-				break;
-			}
-			break;
-		case TEXT_ROTATED_90: /* CW (exact "opposite" of CCW) */
-			switch(direction)
-			{
-			case MULTI_DIR_W:
-				*x = TR_CW_ORIG_X;
-				*y = TR_CW_ORIG_Y - 1 - flf->shadow_offset;
-				x_sign = 0;
-				y_sign = -1;
-				break;
-			case MULTI_DIR_NW:
-				*x = TR_CW_ORIG_X + 1 + flf->shadow_offset;
-				*y = TR_CW_ORIG_Y - 1 - flf->shadow_offset;
-				x_sign = 1;
-				y_sign = -1;
-				break;
-			case MULTI_DIR_N:
-				*x = TR_CW_ORIG_X + 1 + flf->shadow_offset;
-				*y = TR_CW_ORIG_Y;
-				x_sign = 1;
-				y_sign = 0;
-				break;
-			case MULTI_DIR_NE:
-				*x = TR_CW_ORIG_X + 1 + flf->shadow_offset;
-				*y = TR_CW_ORIG_Y + 1 + flf->shadow_offset;
-				x_sign = 1;
-				y_sign = 1;
-				break;
-			case MULTI_DIR_E:
-				*x = TR_CW_ORIG_X;
-				*y = TR_CW_ORIG_Y + 1 + flf->shadow_offset;
-				x_sign = 0;
-				y_sign = 1;
-				break;
-			case MULTI_DIR_SE:
-				*x = TR_CW_ORIG_X - 1 - flf->shadow_offset;
-				*y = TR_CW_ORIG_Y + 1 + flf->shadow_offset;
-				x_sign = -1;
-				y_sign = 1;
-				break;
-			case MULTI_DIR_S:
-				*x = TR_CW_ORIG_X - 1 - flf->shadow_offset;
-				*y = TR_CW_ORIG_Y;
-				x_sign = -1;
-				y_sign = 0;
-				break;
-			case MULTI_DIR_SW:
-				*x = TR_CW_ORIG_X - 1 - flf->shadow_offset;
-				*y = TR_CW_ORIG_Y - 1 - flf->shadow_offset;
-				x_sign = -1;
-				y_sign = -1;
-				break;
-			default: /* never happen */
-				break;
-			}
-			break;
-		case TEXT_ROTATED_0:
-		default: /* no rotation */
-			switch(direction)
-			{
-			case MULTI_DIR_W:
-				*x = NORMAL_ORIG_X - 1 - flf->shadow_offset;
-				*y = NORMAL_ORIG_Y;
-				x_sign = -1;
-				y_sign = 0;
-				break;
-			case MULTI_DIR_NW:
-				*x = NORMAL_ORIG_X - 1 - flf->shadow_offset;
-				*y = NORMAL_ORIG_Y - 1 - flf->shadow_offset;
-				x_sign = -1;
-				y_sign = -1;
-				break;
-			case MULTI_DIR_N:
-				*x = NORMAL_ORIG_X;
-				*y = NORMAL_ORIG_Y - 1 - flf->shadow_offset;
-				x_sign = 0;
-				y_sign = - 1;
-				break;
-			case MULTI_DIR_NE:
-				*x = NORMAL_ORIG_X + 1 + flf->shadow_offset;
-				*y = NORMAL_ORIG_Y - 1 - flf->shadow_offset;
-				x_sign = 1;
-				y_sign = -1;
-				break;
-			case MULTI_DIR_E:
-				*x = NORMAL_ORIG_X + 1 + flf->shadow_offset;
-				*y = NORMAL_ORIG_Y;
-				x_sign = 1;
-				y_sign = 0;
-				break;
-			case MULTI_DIR_SE:
-				*x = NORMAL_ORIG_X + 1 + flf->shadow_offset;
-				*y = NORMAL_ORIG_Y + 1 + flf->shadow_offset;
-				x_sign = 1;
-				y_sign = 1;
-				break;
-			case MULTI_DIR_S:
-				*x = NORMAL_ORIG_X;
-				*y = NORMAL_ORIG_Y + 1 + flf->shadow_offset;
-				x_sign = 0;
-				y_sign = 1;
-				break;
-			case MULTI_DIR_SW:
-				*x = NORMAL_ORIG_X - 1 - flf->shadow_offset;
-				*y = NORMAL_ORIG_Y + 1 + flf->shadow_offset;
-				x_sign = -1;
-				y_sign = 1;
-				break;
-			default: /* never happen */
-				break;
-			}
-			break;
-		}
+		direction_type dir;
+		direction_type dir_x;
+		direction_type dir_y;
+
+		dir = gravity_multi_dir_to_dir(args->direction);
+		gravity_split_xy_dir(&dir_x, &dir_y, dir);
+		args->x_sign = gravity_dir_to_sign_one_axis(dir_x);
+		args->y_sign = gravity_dir_to_sign_one_axis(dir_y);
+		gravity_rotate_xy(
+			args->rot, args->x_sign, args->y_sign,
+			&args->x_sign, &args->y_sign);
+		*x = args->orig_x + args->x_sign * args->offset;
+		*y = args->orig_y + args->y_sign * args->offset;
 	}
-	inter_step++;
-	(*step)++;
+	args->inter_step++;
+	args->step++;
+
 	return True;
 }
 
@@ -1504,7 +1348,8 @@ void FlocaleDrawString(
 	Bool do_free = False;
 	Pixel fg = 0, fgsh = 0;
 	Bool has_fg_pixels = False;
-	
+	flocale_gstp_args gstp_args;
+
 	if (!fws || !fws->str)
 	{
 		return;
@@ -1518,7 +1363,7 @@ void FlocaleDrawString(
 	{
 		len = strlen(fws->str);
 	}
-	
+
 	/* encode the string */
 	FlocaleEncodeWinString(dpy, flf, fws, &do_free, &len);
 
@@ -1530,7 +1375,7 @@ void FlocaleDrawString(
 		has_fg_pixels = True;
 	}
 
-	if (fws->flags.text_rotation != TEXT_ROTATED_0 &&
+	if (fws->flags.text_rotation != ROTATION_0 &&
 	    flf->fftf.fftfont == NULL)
 	{
 		FlocaleRotateDrawString(
@@ -1543,14 +1388,15 @@ void FlocaleDrawString(
 	}
 	else if (flf->fontset != None)
 	{
-		int xt = fws->x, yt = fws->y;
-		int step = 0;
+		int xt = fws->x;
+		int yt = fws->y;
 
+		FlocaleInitGstpArgs(&gstp_args, flf, fws, fws->x, fws->y);
 		if (flf->shadow_size != 0 && has_fg_pixels)
 		{
 			XSetForeground(dpy, fws->gc, fgsh);
-			while(FlocaleGetShadowTextPosition(
-				flf, fws, fws->x, fws->y, &xt, &yt, &step))
+			while (FlocaleGetShadowTextPosition(
+				       &xt, &yt, &gstp_args))
 			{
 				XmbDrawString(
 					dpy, fws->win, flf->fontset, fws->gc,
@@ -1558,6 +1404,8 @@ void FlocaleDrawString(
 			}
 			XSetForeground(dpy, fws->gc, fg);
 		}
+		xt = gstp_args.orig_x;
+		yt = gstp_args.orig_y;
 		XmbDrawString(
 			dpy, fws->win, flf->fontset, fws->gc,
 			xt, yt, fws->e_str, len);
