@@ -8,12 +8,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #ifndef FLOCALE_H
@@ -27,12 +27,13 @@
 #include <X11/Xutil.h>
 
 #include "fvwmlib.h"
+#include "gravity.h"
 #include "Fft.h"
 #include "Colorset.h"
-#include "gravity.h"
 
 /* FlocaleCharset.h and Ficonv.h should not be included */
 
+
 /* ---------------------------- global definitions -------------------------- */
 
 #define FWS_HAVE_LENGTH (1)
@@ -61,80 +62,71 @@
 /* ---------------------------- global macros ------------------------------- */
 
 #define IS_TEXT_DRAWN_VERTICALLY(x) \
-	(x == TEXT_ROTATED_90 || x == TEXT_ROTATED_270)
+	((x) == TEXT_ROTATED_90 || (x) == TEXT_ROTATED_270)
 
-#define FLC_GET_X_CHARSET(fc)	      (fc != NULL)? fc->x:NULL
-#define FLC_SET_ICONV_INDEX(fc, i)    fc->iconv_index = i
-#define FLC_GET_LOCALE_CHARSET(fc, i) fc->locale[i]
+#define FLC_GET_X_CHARSET(fc)         ((fc) != NULL)? (fc)->x:NULL
+#define FLC_SET_ICONV_INDEX(fc, i)    (fc)->iconv_index = i
+#define FLC_GET_LOCALE_CHARSET(fc, i) (fc)->locale[i]
 #define FLC_GET_ICONV_CHARSET(fc) \
-      (fc != NULL && fc->iconv_index >= 0)? fc->locale[fc->iconv_index]:NULL
+	((fc) != NULL && (fc)->iconv_index >= 0)? (fc)->locale[(fc)->iconv_index]:NULL
 #define FLC_DO_ICONV_CHARSET_INITIALIZED(fc) \
-      (fc != NULL && fc->iconv_index != FLC_INDEX_ICONV_CHARSET_NOT_INITIALIZED)
-#define FLC_HAVE_ICONV_CHARSET(fc) (fc != NULL && fc->iconv_index >= 0)
-#define FLC_GET_BIDI_CHARSET(fc) (fc != NULL)? fc->bidi : NULL
+	((fc) != NULL && (fc)->iconv_index != FLC_INDEX_ICONV_CHARSET_NOT_INITIALIZED)
+#define FLC_HAVE_ICONV_CHARSET(fc) ((fc) != NULL && (fc)->iconv_index >= 0)
+#define FLC_GET_BIDI_CHARSET(fc) ((fc) != NULL)? (fc)->bidi : NULL
 #define FLC_ENCODING_TYPE_IS_UTF_8(fc) \
-      (fc != NULL && fc->encoding_type == FLC_ENCODING_TYPE_UTF_8)
+	((fc) != NULL && (fc)->encoding_type == FLC_ENCODING_TYPE_UTF_8)
 #define FLC_ENCODING_TYPE_IS_USC_2(fc) \
-      (fc != NULL && fc->encoding_type == FLC_ENCODING_TYPE_USC_2)
+	((fc) != NULL && (fc)->encoding_type == FLC_ENCODING_TYPE_USC_2)
 #define FLC_ENCODING_TYPE_IS_USC_4(fc) \
-      (fc != NULL && fc->encoding_type == FLC_ENCODING_TYPE_USC_4)
+	((fc) != NULL && (fc)->encoding_type == FLC_ENCODING_TYPE_USC_4)
 
 #define FLC_DEBUG_GET_X_CHARSET(fc)  \
-	 (fc == NULL || fc->x == NULL)? "None":fc->x
+	((fc) == NULL || (fc)->x == NULL)? "None":(fc)->x
 #define FLC_DEBUG_GET_ICONV_CHARSET(fc) \
-	 (fc != NULL && fc->iconv_index >= 0)? fc->locale[fc->iconv_index]:"None"
+	((fc) != NULL && (fc)->iconv_index >= 0)? (fc)->locale[(fc)->iconv_index]:"None"
 #define FLC_DEBUG_GET_BIDI_CHARSET(fc) \
-	 (fc == NULL || fc->bidi == NULL)? "None":fc->bidi
+	((fc) == NULL || (fc)->bidi == NULL)? "None":(fc)->bidi
 
 #define FLF_MULTIDIR_HAS_UPPER(flf) \
-	      ((flf->flags.shadow_dir & MULTI_DIR_NW) || \
-	       (flf->flags.shadow_dir & MULTI_DIR_N) || \
-	       (flf->flags.shadow_dir & MULTI_DIR_NE))
+	(((flf)->flags.shadow_dir & MULTI_DIR_NW) || \
+	 ((flf)->flags.shadow_dir & MULTI_DIR_N) || \
+	 ((flf)->flags.shadow_dir & MULTI_DIR_NE))
 #define FLF_MULTIDIR_HAS_BOTTOM(flf) \
-	      ((flf->flags.shadow_dir & MULTI_DIR_SW) || \
-	       (flf->flags.shadow_dir & MULTI_DIR_S) || \
-	       (flf->flags.shadow_dir & MULTI_DIR_SE))
+	(((flf)->flags.shadow_dir & MULTI_DIR_SW) || \
+	 ((flf)->flags.shadow_dir & MULTI_DIR_S) || \
+	 ((flf)->flags.shadow_dir & MULTI_DIR_SE))
 #define FLF_MULTIDIR_HAS_LEFT(flf) \
-	      ((flf->flags.shadow_dir & MULTI_DIR_SW) || \
-	       (flf->flags.shadow_dir & MULTI_DIR_W) || \
-	       (flf->flags.shadow_dir & MULTI_DIR_NW))
-#define FLF_MULTIDIR_HAS_RIGHT(x) \
-	      ((flf->flags.shadow_dir & MULTI_DIR_SE) || \
-	       (flf->flags.shadow_dir & MULTI_DIR_E) || \
-	       (flf->flags.shadow_dir & MULTI_DIR_NE))
+	(((flf)->flags.shadow_dir & MULTI_DIR_SW) || \
+	 ((flf)->flags.shadow_dir & MULTI_DIR_W) || \
+	 ((flf)->flags.shadow_dir & MULTI_DIR_NW))
+#define FLF_MULTIDIR_HAS_RIGHT(flf) \
+	(((flf)->flags.shadow_dir & MULTI_DIR_SE) || \
+	 ((flf)->flags.shadow_dir & MULTI_DIR_E) || \
+	 ((flf)->flags.shadow_dir & MULTI_DIR_NE))
 
-#define FLF_SHADOW_FULL_SIZE(flf) (flf->shadow_size + flf->shadow_offset)
+#define FLF_SHADOW_FULL_SIZE(flf) ((flf)->shadow_size + (flf)->shadow_offset)
 #define FLF_SHADOW_HEIGHT(flf) \
- (FLF_SHADOW_FULL_SIZE(flf) * \
-	   (FLF_MULTIDIR_HAS_UPPER(flf)+FLF_MULTIDIR_HAS_BOTTOM(flf)))
+	(FLF_SHADOW_FULL_SIZE((flf)) * \
+	 (FLF_MULTIDIR_HAS_UPPER((flf))+FLF_MULTIDIR_HAS_BOTTOM((flf))))
 #define FLF_SHADOW_WIDTH(flf) \
- (FLF_SHADOW_FULL_SIZE(flf) * \
-	    (FLF_MULTIDIR_HAS_LEFT(flf)+FLF_MULTIDIR_HAS_RIGHT(flf)))
+	(FLF_SHADOW_FULL_SIZE((flf)) * \
+	 (FLF_MULTIDIR_HAS_LEFT((flf))+FLF_MULTIDIR_HAS_RIGHT((flf))))
 #define FLF_SHADOW_ASCENT(flf) \
-	 (FLF_SHADOW_FULL_SIZE(flf) * FLF_MULTIDIR_HAS_UPPER(flf))
+	(FLF_SHADOW_FULL_SIZE((flf)) * FLF_MULTIDIR_HAS_UPPER((flf)))
 #define FLF_SHADOW_DESCENT(flf) \
-	 (FLF_SHADOW_FULL_SIZE(flf) * FLF_MULTIDIR_HAS_BOTTOM(flf))
+	(FLF_SHADOW_FULL_SIZE((flf)) * FLF_MULTIDIR_HAS_BOTTOM((flf)))
 
 #define FLF_SHADOW_LEFT_SIZE(flf) \
-	    (FLF_SHADOW_FULL_SIZE(flf) * FLF_MULTIDIR_HAS_LEFT(flf))
+	(FLF_SHADOW_FULL_SIZE((flf)) * FLF_MULTIDIR_HAS_LEFT((flf)))
 #define FLF_SHADOW_RIGHT_SIZE(flf) \
-	    (FLF_SHADOW_FULL_SIZE(flf) * FLF_MULTIDIR_HAS_RIGHT(flf))
+	(FLF_SHADOW_FULL_SIZE((flf)) * FLF_MULTIDIR_HAS_RIGHT((flf)))
 #define FLF_SHADOW_UPPER_SIZE(flf) \
-	    (FLF_SHADOW_FULL_SIZE(flf) * FLF_MULTIDIR_HAS_UPPER(flf))
+	(FLF_SHADOW_FULL_SIZE((flf)) * FLF_MULTIDIR_HAS_UPPER((flf)))
 #define FLF_SHADOW_BOTTOM_SIZE(flf) \
-	    (FLF_SHADOW_FULL_SIZE(flf) * FLF_MULTIDIR_HAS_BOTTOM(flf))
+	(FLF_SHADOW_FULL_SIZE((flf)) * FLF_MULTIDIR_HAS_BOTTOM((flf)))
 
 /* ---------------------------- type definitions ---------------------------- */
 
-typedef enum
-{
-	TEXT_ROTATED_0	  = 0,
-	TEXT_ROTATED_90	  = 1,
-	TEXT_ROTATED_180  = 2,
-	TEXT_ROTATED_270  = 3,
-	TEXT_ROTATED_MASK = 3,
-} text_rotation_type;
-
 typedef struct FlocaleCharset
 {
 	char *x;               /* X font charset */
@@ -149,12 +141,12 @@ typedef struct _FlocaleFont
 	struct _FlocaleFont *next;
 	char *name;
 	int count;
-	XFontStruct *font;	/* font structure */
-	XFontSet fontset;	/* font set */
+	XFontStruct *font;      /* font structure */
+	XFontSet fontset;       /* font set */
 	FftFontType fftf;       /* fvwm xft font */
 	FlocaleCharset *fc;     /* fvwm charset of the font */
 	FlocaleCharset *str_fc; /* fvwm charset of the strings to be displayed */
-	int height;		/* height of the font: ascent + descent */
+	int height;             /* height of the font: ascent + descent */
 	int ascent;
 	int descent;
 	int max_char_width;
@@ -162,7 +154,7 @@ typedef struct _FlocaleFont
 	short shadow_offset;
 	struct
 	{
-		unsigned shadow_dir : 8;
+		unsigned shadow_dir : (DIR_ALL_MASK + 1);
 		unsigned must_free_fc : 1;
 		/* is_mb are used only with a XFontStruct font, for XFontSet
 		 * everything is done in the good way automatically and this
@@ -177,15 +169,17 @@ typedef struct
 	char *e_str;    /* tmp */
 	XChar2b *str2b; /* tmp */
 	GC gc;
-	colorset_struct *colorset;
+	colorset_t *colorset;
 	Window win;
 	int x;
 	int y;
 	int len;
+	Region	clip_region;
 	struct
 	{
 		unsigned text_rotation : 2;
 		unsigned has_colorset : 1;
+		unsigned has_clip_region : 1;
 	} flags;
 } FlocaleWinString;
 
@@ -195,6 +189,23 @@ typedef struct
 	char **name_list;
 } FlocaleNameString;
 
+typedef struct
+{
+	int step;
+	int orig_x;
+	int orig_y;
+	int offset;
+	int outer_offset;
+	multi_direction_type direction;
+	int inter_step;
+	int num_inter_steps;
+	int x_sign;
+	int y_sign;
+	int size;
+	unsigned sdir : (DIR_ALL_MASK + 1);
+	rotation_type rot;
+} flocale_gstp_args;
+
 /* ---------------------------- exported variables (globals) ---------------- */
 
 /* ---------------------------- interface functions ------------------------- */
@@ -222,7 +233,7 @@ void FlocaleInit(
 /*
  * load a FlocaleFont (create it or load it from a cache)
  * fontname: a ";" sperated list of "," separated list of XFLD font names or
- * either "xft:" followed by a	Xft font name. Examples:
+ * either "xft:" followed by a  Xft font name. Examples:
  * "xft:Verdana:Bold:pixelsize=14:rgba=rgb"
  * "xft:Verdana:size=12;-adobe-courier-medium-r-normal--14-*,fixed"
  * module: name of the fvwm module for errors msg
@@ -232,12 +243,12 @@ void FlocaleInit(
  * 1) We try to load each element "fn" of the ";" seprated list until success
  *    as follows:
  *    a - if fn begin with "xft:", then if FftSupport fn is loaded as an xft
- *	  font; if !FftSupport fn is skipped (ignored)
+ *        font; if !FftSupport fn is skipped (ignored)
  *    b - If the locale is supported fn is loaded using XCreateFontSet. If this
- *	  fail fallback into 1-c)
+ *        fail fallback into 1-c)
  *    c - If the locale is not supported or 1-b fail fn is loaded using
- *	  XLoadQueryFont (the first loadable font in the fn "," separated list
- *	  is load)
+ *        XLoadQueryFont (the first loadable font in the fn "," separated list
+ *        is load)
  * 2) If 0) and 1) fail:
  *    - try to load MB_FALLBACK_FONT with XCreateFontSet
  *    - If this fail try to load FALLBACK_FONT with XLoadQueryFont
@@ -282,9 +293,13 @@ void FlocaleDrawUnderline(
 /*
  * Get the position for shadow text
  */
-Bool FlocaleGetShadowTextPosition(FlocaleFont *flf, FlocaleWinString *fws,
-			    int orig_x, int orig_y,
-			    int *x, int *y, int *step);
+void FlocaleInitGstpArgs(
+	flocale_gstp_args *args, FlocaleFont *flf, FlocaleWinString *fws,
+	int start_x, int start_y);
+
+Bool FlocaleGetShadowTextPosition(
+	int *x, int *y, flocale_gstp_args *args);
+
 /*
  * Call XmbTextEscapement(ff->fontset, str, sl) if ff->fontset is not None.
  * Call XTextWith(ff->font, str, sl) if ff->font is not NULL.
