@@ -20,16 +20,16 @@
 
 #include "config.h"
 
+#include <X11/Xlib.h>
+#include <stdio.h>
+#include <math.h>
+
 #include "defaults.h"
 #include "libs/fvwmlib.h"
 #include "libs/PictureBase.h"
 #include "libs/PictureUtils.h"
 #include "libs/PictureGraphics.h"
 
-#include <X11/Xlib.h>
-#include <stdio.h>
-#include <math.h>
-
 /* ---------------------------- local definitions --------------------------- */
 
 /* ---------------------------- local macros -------------------------------- */
@@ -69,11 +69,6 @@ void do_relieve_rectangle(
 
 	if (w <= 0 || h <= 0)
 	{
-		if (w < 0 || h < 0)
-		{
-			fprintf(stderr, "RelieveRectangle:"
-				" called with w = %d, h = %d\n", w, h);
-		}
 		return;
 	}
 	seg = (XSegment*)alloca((sizeof(XSegment) * line_width) * 2);
@@ -114,6 +109,7 @@ Pixmap CreateStretchXPixmap(
 {
 	int i;
 	Pixmap pixmap;
+	GC my_gc = None;
 
 	if (src_width < 0 || src_height < 0 || dest_width < 0)
 	{
@@ -124,13 +120,20 @@ Pixmap CreateStretchXPixmap(
 	{
 		return None;
 	}
+	if (gc == None)
+	{
+		my_gc = fvwmlib_XCreateGC(dpy, pixmap, 0, 0);
+	}
 	for (i = 0; i < dest_width; i++)
 	{
 		XCopyArea(
-			dpy, src, pixmap, gc, (i * src_width) / dest_width, 0,
-			1, src_height, i, 0);
+			dpy, src, pixmap, (gc == None)? my_gc:gc,
+			(i * src_width) / dest_width, 0, 1, src_height, i, 0);
+	}
+	if (my_gc)
+	{
+		XFreeGC(dpy, my_gc);
 	}
-
 	return pixmap;
 }
 
@@ -144,6 +147,7 @@ Pixmap CreateStretchYPixmap(
 {
 	int i;
 	Pixmap pixmap;
+	GC my_gc = None;
 
 	if (src_height < 0 || src_depth < 0 || dest_height < 0)
 	{
@@ -154,13 +158,20 @@ Pixmap CreateStretchYPixmap(
 	{
 		return None;
 	}
+	if (gc == None)
+	{
+		my_gc = fvwmlib_XCreateGC(dpy, pixmap, 0, 0);
+	}
 	for (i = 0; i < dest_height; i++)
 	{
 		XCopyArea(
-			dpy, src, pixmap, gc, 0, (i * src_height) / dest_height,
-			src_width, 1, 0, i);
+			dpy, src, pixmap, (gc == None)? my_gc:gc,
+			0, (i * src_height) / dest_height, src_width, 1, 0, i);
+	}
+	if (my_gc)
+	{
+		XFreeGC(dpy, my_gc);
 	}
-
 	return pixmap;
 }
 
@@ -174,23 +185,35 @@ Pixmap CreateStretchPixmap(
 {
 	Pixmap pixmap = None;
 	Pixmap temp_pixmap;
+	GC my_gc = None;
 
 	if (src_width < 0 || src_height < 0 || src_depth < 0 || dest_width < 0)
 	{
 		return None;
 	}
-
+	if (gc == None)
+	{
+		my_gc = fvwmlib_XCreateGC(dpy, src, 0, 0);
+	}
 	temp_pixmap = CreateStretchXPixmap(
-		dpy, src, src_width, src_height, src_depth, dest_width, gc);
+		dpy, src, src_width, src_height, src_depth, dest_width,
+		(gc == None)? my_gc:gc);
 	if (temp_pixmap == None)
 	{
+		if (my_gc)
+		{
+			XFreeGC(dpy, my_gc);
+		}
 		return None;
 	}
 	pixmap = CreateStretchYPixmap(
 		dpy, temp_pixmap, dest_width, src_height, src_depth,
 		dest_height, gc);
 	XFreePixmap(dpy, temp_pixmap);
-
+	if (my_gc)
+	{
+		XFreeGC(dpy, my_gc);
+	}
 	return pixmap;
 }
 
@@ -397,9 +420,13 @@ XColor *AllocNonlinearGradient(
 		int skip_first_color = (curpixel != 0);
 
 		if (i == 0)
+		{
 			n = seg_end_colors[0] + 1;
+		}
 		else
+		{
 			n = seg_end_colors[i] - seg_end_colors[i - 1] + 1;
+		}
 
 		if (n > 1)
 		{
@@ -412,8 +439,9 @@ XColor *AllocNonlinearGradient(
 				return NULL;
 			}
 			for (j = skip_first_color; j < n; ++j)
+			{
 				xcs[curpixel + j] = c[j];
-			
+			}
 			curpixel += n - 1;
 		}
 		if (c)
@@ -450,14 +478,16 @@ XColor *AllocAllGradientColors(
 	for (i = 0; i <= nsegs; i++)
 	{
 		if (color_names[i])
+		{
 			free(color_names[i]);
+		}
 	}
 	free(color_names);
 	free(perc);
 	if (!xcs)
 	{
 		fprintf(stderr, "couldn't create gradient\n");
-		return None;
+		return NULL;
 	}
 
 	return xcs;
@@ -619,7 +649,7 @@ Bool CalculateGradientDimensions(
 	unsigned int *width_ret, unsigned int *height_ret)
 {
 	static unsigned int best_width = 0, best_height = 0;
-	int dither_factor = (dither > 0)? ncolors:1;
+	int dither_factor = (dither > 0)? 128:1;
 
 	/* get the best tile size (once) */
 	if (!best_width)
@@ -706,19 +736,16 @@ Drawable CreateGradientPixmap(
 	int t_y;
 	int t_width;
 	int t_height;
+	int ps;
 
-	if (d_pixels != NULL)
+	if (d_pixels != NULL && *d_pixels != NULL)
 	{
-		if (*d_pixels != NULL)
+		if (d_npixels != NULL && *d_npixels > 0)
 		{
-			if (d_npixels != NULL && d_npixels > 0)
-			{
-				PictureFreeColors(
-					dpy, Pcmap, *d_pixels, *d_npixels, 0,
-					False);
-			}
-			free(*d_pixels);
+			PictureFreeColors(
+				dpy, Pcmap, *d_pixels, *d_npixels, 0, False);
 		}
+		free(*d_pixels);
 		*d_pixels = NULL;
 	}
 	if (d_npixels != NULL)
@@ -766,6 +793,7 @@ Drawable CreateGradientPixmap(
 	{
 		pixels = (Pixel *)safemalloc(t_width * t_height * sizeof(Pixel));
 	}
+	ps = t_width * t_height;
 	/* now do the fancy drawing */
 	switch (type)
 	{
@@ -824,7 +852,7 @@ Drawable CreateGradientPixmap(
 				if (dither)
 				{
 					PictureAllocColorAllProp(
-						dpy, Pcmap, &c, i,j, False, 
+						dpy, Pcmap, &c, i,j, False,
 						False,True);
 					if (PUseDynamicColors)
 						pixels[k++] = c.pixel;
@@ -841,7 +869,7 @@ Drawable CreateGradientPixmap(
 		{
 			for (j = 0; j < t_height; j++)
 			{
-				c = xcs[(i + (t_height - j - 1)) * ncolors / 
+				c = xcs[(i + (t_height - j - 1)) * ncolors /
 				       t_scale];
 				if (dither)
 				{
@@ -899,7 +927,7 @@ Drawable CreateGradientPixmap(
 				if (dither)
 				{
 					PictureAllocColorAllProp(
-						dpy, Pcmap, &c, i,j, False, 
+						dpy, Pcmap, &c, i,j, False,
 						False,True);
 					if (PUseDynamicColors)
 						pixels[k++] = c.pixel;
@@ -1005,14 +1033,22 @@ Drawable CreateGradientPixmap(
 		 * color */
 		memset(image->data, 0, image->bytes_per_line * t_height);
 		XAddPixel(image, xcs[0].pixel);
+		if (dither && PUseDynamicColors)
+		{
+			free(pixels);
+			pixels = (Pixel *)safemalloc(sizeof(Pixel));
+			pixels[0] = xcs[0].pixel;
+			ps = 1;
+		}
 		break;
 	}
 
 	if (dither && PUseDynamicColors)
 	{
-		if (d_pixels != NULL)
+		if (d_pixels != NULL && d_npixels != NULL)
 		{
 			*d_pixels = pixels;
+			*d_npixels = ps;
 		}
 		else
 		{
@@ -1054,7 +1090,7 @@ Pixmap CreateGradientPixmapFromString(
 	Pixel **pixels_return, int *nalloc_pixels, int dither)
 {
 	Pixel *d_pixels = NULL;
-	unsigned int d_npixels; 
+	unsigned int d_npixels = 0;
 	XColor *xcs = NULL;
 	unsigned int ncolors = 0;
 	char **colors;
@@ -1094,47 +1130,50 @@ Pixmap CreateGradientPixmapFromString(
 
 	/* if the caller has not asked for the pixels there is probably a leak
 	 */
-	if (!pixels_return && PUseDynamicColors)
+	if (PUseDynamicColors)
 	{
-		fprintf(stderr,
+		if (!(pixels_return && nalloc_pixels))
+		{
+			/* if the caller has not asked for the pixels there is
+			 * probably a leak */
+			fprintf(stderr,
 			"CreateGradient: potential color leak, losing track"
 			" of pixels\n");
-	}
-	else if (PUseDynamicColors)
-	{
-		if (!dither)
-		{
-			Pixel *pixels;
-			int i;
-
-			pixels = (Pixel *)safemalloc(ncolors * sizeof(Pixel));
-			for(i=0; i<ncolors; i++)
+			if (d_pixels != NULL)
 			{
-				pixels[i] = xcs[i].pixel;
+				free(d_pixels);
 			}
-			*pixels_return = pixels;
 		}
 		else
 		{
-			*pixels_return = d_pixels;
+			if (!dither)
+			{
+				Pixel *pixels;
+				int i;
+
+				pixels = (Pixel *)safemalloc(
+					ncolors * sizeof(Pixel));
+				for(i=0; i<ncolors; i++)
+				{
+					pixels[i] = xcs[i].pixel;
+				}
+				*pixels_return = pixels;
+				*nalloc_pixels = ncolors;
+			}
+			else
+			{
+				*pixels_return = d_pixels;
+				*nalloc_pixels = d_npixels;
+			}
 		}
 	}
 	else if (d_pixels != NULL)
 	{
+		/* should not happen */
 		free(d_pixels);
 	}
 
-	if (nalloc_pixels && PUseDynamicColors)
-	{
-		if (dither == 0)
-		{
-			*nalloc_pixels = ncolors;
-		}
-		else
-		{
-			*nalloc_pixels = d_npixels;
-		}
-	}
+	free(xcs);
 
 	return pixmap;
 }
@@ -1288,7 +1327,9 @@ GC fvwmlib_XCreateGC(
 	XGCValues gcv;
 
 	if (!values)
+	{
 		values = &gcv;
+	}
 	f = values->graphics_exposures;
 	if (!(valuemask & GCGraphicsExposures))
 	{
