@@ -5,12 +5,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 /****************************************************************************
@@ -32,7 +32,7 @@
  * as long as the copyright is kept intact.
  *
  * Romano Giannetti - Dipartimento di Ingegneria dell'Informazione
- *		      via Diotisalvi, 2	 PISA
+ *                    via Diotisalvi, 2  PISA
  * mailto:romano@iet.unipi.it
  * http://www.iet.unipi.it/~romano
  *
@@ -51,7 +51,7 @@
 #include <signal.h>
 #include <ctype.h>
 
-#include <sys/time.h>
+#include "ftime.h"
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -60,21 +60,23 @@
 #include "fvwmlib.h"
 #include "Colorset.h"
 #include "Picture.h"
+#include "PictureUtils.h"
 
 static FvwmPicture *FvwmPictureList=NULL;
 
-FvwmPicture *PGetFvwmPicture(Display *dpy, Window Root, char *ImagePath,
-			     char *name, int color_limit)
+FvwmPicture *PGetFvwmPicture(
+	Display *dpy, Window win, char *ImagePath, const char *name,
+	FvwmPictureAttributes fpa)
 {
-	char *path = PictureFindImageFile( name, ImagePath, R_OK );
+	char *path = PictureFindImageFile(name, ImagePath, R_OK);
 	FvwmPicture *p;
 
-	if ( path == NULL )
+	if (path == NULL)
 	{
 		return NULL;
 	}
-	p = PImageLoadFvwmPictureFromFile(dpy, Root, path, color_limit);
-	if ( p == NULL )
+	p = PImageLoadFvwmPictureFromFile(dpy, win, path, fpa);
+	if (p == NULL)
 	{
 		free(path);
 	}
@@ -82,14 +84,15 @@ FvwmPicture *PGetFvwmPicture(Display *dpy, Window Root, char *ImagePath,
 	return p;
 }
 
-FvwmPicture *PCacheFvwmPicture(Display *dpy, Window Root, char *ImagePath,
-			       char *name, int color_limit)
+FvwmPicture *PCacheFvwmPicture(
+	Display *dpy, Window win, char *ImagePath, const char *name,
+	FvwmPictureAttributes fpa)
 {
 	char *path;
 	FvwmPicture *p = FvwmPictureList;
 
 	/* First find the full pathname */
-	if ( !(path = PictureFindImageFile(name,ImagePath,R_OK)) )
+	if ((path = PictureFindImageFile(name, ImagePath, R_OK)) == NULL)
 	{
 		return NULL;
 	}
@@ -99,7 +102,7 @@ FvwmPicture *PCacheFvwmPicture(Display *dpy, Window Root, char *ImagePath,
 	{
 		register char *p1, *p2;
 
-		for (p1=path, p2=p->name; *p1 && *p2; ++p1, ++p2)
+		for (p1 = path, p2 = p->name; *p1 && *p2; ++p1, ++p2)
 		{
 			if (*p1 != *p2)
 			{
@@ -108,7 +111,8 @@ FvwmPicture *PCacheFvwmPicture(Display *dpy, Window Root, char *ImagePath,
 		}
 
 		/* If we have found a picture with the wanted name and stamp */
-		if (!*p1 && !*p2 && !isFileStampChanged(&p->stamp, p->name))
+		if (!*p1 && !*p2 && !isFileStampChanged(&p->stamp, p->name) &&
+		    PICTURE_FPA_AGREE(p,fpa))
 		{
 			p->count++; /* Put another weight on the picture */
 			free(path);
@@ -119,7 +123,7 @@ FvwmPicture *PCacheFvwmPicture(Display *dpy, Window Root, char *ImagePath,
 
 	/* Not previously cached, have to load it ourself. Put it first in list
 	 */
-	p = PImageLoadFvwmPictureFromFile(dpy, Root, path, color_limit);
+	p = PImageLoadFvwmPictureFromFile(dpy, win, path, fpa);
 	if(p)
 	{
 		p->next=FvwmPictureList;
@@ -153,9 +157,9 @@ void PDestroyFvwmPicture(Display *dpy, FvwmPicture *p)
 	{
 		if (p->nalloc_pixels != 0)
 		{
-			XFreeColors(
+			PictureFreeColors(
 				dpy, Pcmap, p->alloc_pixels, p->nalloc_pixels,
-				0);
+				0, False);
 		}
 		free(p->alloc_pixels);
 	}
@@ -196,9 +200,9 @@ void PDestroyFvwmPicture(Display *dpy, FvwmPicture *p)
 	return;
 }
 
-FvwmPicture *PLoadFvwmPictureFromPixmap(Display *dpy, Window Root, char *name,
-					Pixmap pixmap, Pixmap mask, Pixmap alpha,
-					int width, int height)
+FvwmPicture *PLoadFvwmPictureFromPixmap(
+	Display *dpy, Window win, char *name, Pixmap pixmap,
+	Pixmap mask, Pixmap alpha, int width, int height)
 {
 	FvwmPicture *q;
 
@@ -220,9 +224,9 @@ FvwmPicture *PLoadFvwmPictureFromPixmap(Display *dpy, Window Root, char *name,
 	return q;
 }
 
-FvwmPicture *PCacheFvwmPictureFromPixmap(Display *dpy, Window Root, char *name,
-					 Pixmap pixmap, Pixmap mask,
-					 Pixmap alpha, int width, int height)
+FvwmPicture *PCacheFvwmPictureFromPixmap(
+	Display *dpy, Window win, char *name, Pixmap pixmap,
+	Pixmap mask, Pixmap alpha, int width, int height)
 {
 	FvwmPicture *p = FvwmPictureList;
 
@@ -240,7 +244,7 @@ FvwmPicture *PCacheFvwmPictureFromPixmap(Display *dpy, Window Root, char *name,
 	}
 
 	/* Not previously cached, have to load. Put it first in list */
-	p = PLoadFvwmPictureFromPixmap(dpy, Root, name, pixmap, mask,
+	p = PLoadFvwmPictureFromPixmap(dpy, win, name, pixmap, mask,
 				       alpha, width, height);
 	if(p)
 	{
