@@ -5,12 +5,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 /*
@@ -22,7 +22,6 @@
 
 #include <stdio.h>
 #include <signal.h>
-#include <string.h>
 #include <ctype.h>
 #include <unistd.h>
 
@@ -122,7 +121,7 @@ void fvwmlib_keyboard_shortcuts(
     int x_def_new = 0;
     int y_def_new = 0;
 
-    if (XQueryPointer(
+    if (FQueryPointer(
 	  dpy, RootWindow(dpy, screen), &JunkRoot, &Event->xany.window,
 	  &x_root, &y_root, &x, &y, &JunkMask) == False)
     {
@@ -183,7 +182,7 @@ void fvwmlib_keyboard_shortcuts(
     }
     if (x_move || y_move)
     {
-      XWarpPointer(dpy, None, RootWindow(dpy, screen), 0, 0, 0, 0,
+      FWarpPointer(dpy, None, RootWindow(dpy, screen), 0, 0, 0, 0,
 		   x_root + x_move, y_root + y_move);
     }
     /* beat up the event */
@@ -242,7 +241,7 @@ void fvwmlib_get_target_window(
 
   while (!finished && !canceled)
   {
-    XMaskEvent(dpy, ButtonPressMask | ButtonReleaseMask |
+    FMaskEvent(dpy, ButtonPressMask | ButtonReleaseMask |
 		   KeyPressMask | KeyReleaseMask, &eventp);
     switch (eventp.type)
     {
