@@ -53,7 +53,7 @@
 #include "libs/PictureBase.h"
 #include "libs/fvwmsignal.h"
 #include "libs/FScreen.h"
-
+#include "libs/FRenderInit.h"
 
 #include "fvwmDragWell.h"
 #include "dragSource.h"
@@ -444,6 +444,7 @@ void XStartup(char *appName)
   FScreenInit(xg.dpy);
   AllocColorset(0);
   FShapeInit(xg.dpy);
+  FRenderInit(xg.dpy);
 
   /*get X stuff*/
   xg.xfd = XConnectionNumber(xg.dpy);
@@ -752,9 +753,9 @@ int myXNextEvent(XEvent *event, char *fvwmMessage)
   char *msg, *tline, *token;
   unsigned long *lptr;
   int colorset;
-  if(XPending(xg.dpy)) /*look for an X event first*/
+  if(FPending(xg.dpy)) /*look for an X event first*/
   {
-    XNextEvent(xg.dpy,event); /*got an X event*/
+    FNextEvent(xg.dpy,event); /*got an X event*/
     return FOUND_XEVENT;
   }
 
@@ -765,9 +766,9 @@ int myXNextEvent(XEvent *event, char *fvwmMessage)
   {
     if(FD_ISSET(xg.xfd, &in_fdset))
     {
-      if(XPending(xg.dpy))
+      if(FPending(xg.dpy))
       {
-	XNextEvent(xg.dpy,event); /*get an X event*/
+	FNextEvent(xg.dpy,event); /*get an X event*/
 	miss_counter = 0;
 	return FOUND_XEVENT;
       }
