@@ -5,12 +5,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "config.h"
@@ -73,8 +73,8 @@ typedef union {
   ConfigWinPacket      add_config_data;
   m_new_page_data      new_page_data;
   m_minimal_data       minimal_data;
-  m_icon_data	       icon_data;
-  m_name_data	       name_data;
+  m_icon_data          icon_data;
+  m_name_data          name_data;
   m_mini_icon_data     mini_icon_data;
   m_property_data      property_data;
 } FvwmPacketBody;
@@ -198,6 +198,7 @@ int win_in_viewport (WinData *win)
 static WinData *id_to_win (Ulong id)
 {
   WinData *win;
+
   win = find_win_hashtab (id);
   if (win == NULL) {
     win = new_windata ();
@@ -205,6 +206,7 @@ static WinData *id_to_win (Ulong id)
     win->app_id_set = 1;
     insert_win_hashtab (win);
   }
+
   return win;
 }
 
@@ -215,6 +217,8 @@ static void set_win_configuration (WinData *win, FvwmPacketBody *body)
   win->y = body->add_config_data.frame_y;
   win->width = body->add_config_data.frame_width;
   win->height = body->add_config_data.frame_height;
+  win->iconified =  IS_ICONIFIED(&(body->add_config_data));
+  win->state = PLAIN_CONTEXT;
   win->geometry_set = 1;
   memcpy(&(win->flags), &(body->add_config_data.flags), sizeof(win->flags));
 }
@@ -269,7 +273,7 @@ static void focus_change (FvwmPacketBody *body)
   ConsoleDebug (FVWM, "Focus Change\n");
   ConsoleDebug (FVWM, "\tID: %ld\n", app_id);
 
-  if (fvwm_focus_win)
+  if (fvwm_focus_win && win != fvwm_focus_win)
   {
     del_win_state (fvwm_focus_win, FOCUS_CONTEXT);
     if (fvwm_focus_win->manager && fvwm_focus_win->manager->focus_button)
@@ -283,8 +287,8 @@ static void focus_change (FvwmPacketBody *body)
       win->manager->window_up  &&
       win->manager->followFocus) {
     win->manager->focus_button = win->button;
-    add_win_state (win, FOCUS_CONTEXT);
   }
+  add_win_state (win, FOCUS_CONTEXT);
 
   globals.focus_win = win;
   fvwm_focus_win = win;
@@ -358,7 +362,8 @@ static void visible_icon_name (FvwmPacketBody *body)
 
   win = id_to_win (app_id);
 
-  if (win->visible_icon_name && !strcmp (win->visible_icon_name, (char *)name)) {
+  if (win->visible_icon_name && !strcmp (win->visible_icon_name, (char *)name))
+  {
     ConsoleDebug (FVWM, "No icon change: %s %s\n", win->iconname, name);
     return;
   }
@@ -434,22 +439,12 @@ static void new_window (FvwmPacketBody *body)
 {
   WinData *win;
 
-  win = new_windata();
+  win = id_to_win (body->add_config_data.w);
   memcpy(&(win->flags), &(body->add_config_data.flags), sizeof(win->flags));
-  if (!(IS_TRANSIENT(win)))
-  {
-    win->app_id = body->add_config_data.w;
-    win->app_id_set = 1;
-    set_win_configuration (win, body);
-
-    insert_win_hashtab (win);
-    check_win_complete (win);
-    check_in_window (win);
-  }
-  else
-  {
-    Free(win);
-  }
+  set_win_configuration (win, body);
+  got_configure (win->manager);
+  check_win_complete (win);
+  check_in_window (win);
 }
 
 static void destroy_window (FvwmPacketBody *body)
@@ -540,34 +535,42 @@ static void sendtomodule (FvwmPacketBody *body)
 
 static void property_change (FvwmPacketBody *body)
 {
-  WinManager *man = NULL;
-  int j;
+	WinManager *man = NULL;
+	int j;
 
-  if (body->property_data.id != 0)
-  {
-    if (!(man = find_windows_manager(body->property_data.id)))
-      return;
-  }
+	if (body->property_data.id != 0)
+	{
+		if (!(man = find_windows_manager(body->property_data.id)))
+			return;
+	}
 
-  if (body->property_data.arg == MX_PROPERTY_CHANGE_BACKGROUND)
-  {
-    if (man != NULL && man->pixmap[DEFAULT] == ParentRelative)
-    {
-      XClearArea(theDisplay, man->theWindow, 0, 0, 0, 0, True);
-    }
-    if (man != NULL)
-      return;
-    for (j = 0; j < globals.num_managers; j++)
-    {
-      man = &globals.managers[j];
-      if (man->pixmap[DEFAULT] == ParentRelative)
-	XClearArea(theDisplay, man->theWindow, 0, 0, 0, 0, True);
-    }
-  }
-  else if  (body->property_data.arg == MX_PROPERTY_CHANGE_SWALLOW && man != NULL)
-  {
-    man->swallowed = body->property_data.toggle;
-  }
+	if (body->property_data.arg == MX_PROPERTY_CHANGE_BACKGROUND)
+	{
+		if (man != NULL &&
+		    CSET_IS_TRANSPARENT_PR(man->colorsets[DEFAULT]))
+		{
+			recreate_background(man, DEFAULT);
+			force_manager_redraw(man);
+		}
+		if (man != NULL)
+		{
+			return;
+		}
+		for (j = 0; j < globals.num_managers; j++)
+		{
+			man = &globals.managers[j];
+			if (CSET_IS_TRANSPARENT_PR(man->colorsets[DEFAULT]))
+			{
+				recreate_background(man, DEFAULT);
+				force_manager_redraw(man);
+			}
+		}
+	}
+	else if  (body->property_data.arg == MX_PROPERTY_CHANGE_SWALLOW &&
+		  man != NULL)
+	{
+		man->swallowed = body->property_data.toggle;
+	}
 }
 
 static void ProcessMessage (Ulong type, FvwmPacketBody *body)
