@@ -5,12 +5,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #ifndef IN_X_H
@@ -30,5 +30,6 @@ extern void xevent_loop (void);
 extern void create_manager_window (int man_id);
 extern void X_init_manager (int man_id);
 extern void change_colorset(int color);
-
+extern void recreate_background(WinManager *man, Contexts i);
+extern void recreate_transparent_bgs(WinManager *man);
 #endif
