@@ -14,12 +14,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "config.h"
@@ -28,7 +28,7 @@
 #include <signal.h>
 #include <fcntl.h>
 #include <sys/wait.h>
-#include <sys/time.h>
+#include "libs/ftime.h"
 #include <ctype.h>
 
 #ifdef HAVE_SYS_BSDTYPES_H
@@ -75,7 +75,7 @@ ScreenInfo Scr;
 PagerWindow *Start = NULL;
 PagerWindow *FocusWin = NULL;
 
-Display *dpy;			/* which display are we talking to */
+Display *dpy;                   /* which display are we talking to */
 int x_fd;
 fd_set_size_t fd_width;
 
@@ -159,7 +159,7 @@ void ExitPager(void);
 /***********************************************************************
  *
  *  Procedure:
- *	main - start of module
+ *      main - start of module
  *
  ***********************************************************************/
 int main(int argc, char **argv)
@@ -355,7 +355,7 @@ int main(int argc, char **argv)
   ParseOptions();
   if (is_transient)
   {
-    if (XQueryPointer(
+    if (FQueryPointer(
 	  dpy, Scr.Root, &JunkRoot, &JunkChild, &window_x, &window_y, &JunkX,
 	  &JunkY, &JunkMask) == False)
     {
@@ -466,7 +466,7 @@ int main(int argc, char **argv)
 /***********************************************************************
  *
  *  Procedure:
- *	Loop - wait for data to process
+ *      Loop - wait for data to process
  *
  ***********************************************************************/
 void Loop(int *fd)
@@ -499,7 +499,7 @@ void Loop(int *fd)
 /***********************************************************************
  *
  *  Procedure:
- *	Process message - examines packet types, and takes appropriate action
+ *      Process message - examines packet types, and takes appropriate action
  *
  ***********************************************************************/
 void process_message( FvwmPacket* packet )
@@ -573,7 +573,7 @@ void process_message( FvwmPacket* packet )
 /***********************************************************************
  *
  *  Procedure:
- *	SIGPIPE handler - SIGPIPE means fvwm is dying
+ *      SIGPIPE handler - SIGPIPE means fvwm is dying
  *
  ***********************************************************************/
 static RETSIGTYPE
@@ -590,7 +590,7 @@ void DeadPipe(int nonsense)
 /***********************************************************************
  *
  *  Procedure:
- *	list_add - displays packet contents to stderr
+ *      list_add - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_add(unsigned long *body)
@@ -636,8 +636,8 @@ void list_add(unsigned long *body)
   }
   else
   {
-    (*prev)->text = body[22];
-    (*prev)->back = body[23];
+    (*prev)->text = cfgpacket->TextPixel;
+    (*prev)->back = cfgpacket->BackPixel;
   }
   AddNewWindow(*prev);
 }
@@ -645,7 +645,7 @@ void list_add(unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_configure - displays packet contents to stderr
+ *      list_configure - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_configure(unsigned long *body)
@@ -685,8 +685,8 @@ void list_configure(unsigned long *body)
   }
   else
   {
-    t->text = body[22];
-    t->back = body[23];
+    t->text = cfgpacket->TextPixel;
+    t->back = cfgpacket->BackPixel;
   }
   if (IS_ICONIFIED(t))
   {
@@ -720,7 +720,7 @@ void list_configure(unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_destroy - displays packet contents to stderr
+ *      list_destroy - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_destroy(unsigned long *body)
@@ -761,7 +761,7 @@ void list_destroy(unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_focus - displays packet contents to stderr
+ *      list_focus - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_focus(unsigned long *body)
@@ -808,7 +808,7 @@ void list_focus(unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_new_page - displays packet contents to stderr
+ *      list_new_page - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_new_page(unsigned long *body)
@@ -839,7 +839,7 @@ void list_new_page(unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_new_desk - displays packet contents to stderr
+ *      list_new_desk - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_new_desk(unsigned long *body)
@@ -966,7 +966,6 @@ void list_new_desk(unsigned long *body)
     {
       change_colorset(change_highcs);
     }
-
     XClearWindow(dpy, Desks[0].w);
     XClearWindow(dpy, Desks[0].title_w);
   } /* if (fAlwaysCurrentDesk && oldDesk != Scr.CurrentDesk) */
@@ -986,13 +985,13 @@ void list_new_desk(unsigned long *body)
       sprintf(line, "Desk %d", Scr.CurrentDesk);
       name = &(line[0]);
     }
-      XStoreName(dpy, Scr.Pager_w, name);
-      XSetIconName(dpy, Scr.Pager_w, name);
+    XStoreName(dpy, Scr.Pager_w, name);
+    XSetIconName(dpy, Scr.Pager_w, name);
   }
 
   MovePage(True);
-  DrawGrid(oldDesk - desk1,1);
-  DrawGrid(Scr.CurrentDesk - desk1,1);
+  DrawGrid(oldDesk - desk1, 1, None, NULL);
+  DrawGrid(Scr.CurrentDesk - desk1, 1, None, NULL);
   MoveStickyWindows();
 /*
   Hilight(FocusWin,False);
@@ -1003,7 +1002,7 @@ void list_new_desk(unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_raise - displays packet contents to stderr
+ *      list_raise - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_raise(unsigned long *body)
@@ -1029,7 +1028,7 @@ void list_raise(unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_lower - displays packet contents to stderr
+ *      list_lower - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_lower(unsigned long *body)
@@ -1057,7 +1056,7 @@ void list_lower(unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_iconify - displays packet contents to stderr
+ *      list_iconify - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_iconify(unsigned long *body)
@@ -1109,7 +1108,7 @@ void list_iconify(unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_deiconify - displays packet contents to stderr
+ *      list_deiconify - displays packet contents to stderr
  *
  ***********************************************************************/
 
@@ -1212,7 +1211,7 @@ void list_window_name(unsigned long *body,unsigned long type)
 /***********************************************************************
  *
  *  Procedure:
- *	list_icon_name - displays packet contents to stderr
+ *      list_icon_name - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_icon_name(unsigned long *body)
@@ -1245,7 +1244,7 @@ void list_icon_name(unsigned long *body)
 
 void list_mini_icon(unsigned long *body)
 {
-  PagerWindow	*t;
+  PagerWindow   *t;
   Window target_w;
   MiniIconPacket *mip = (MiniIconPacket *) body;
 
@@ -1255,12 +1254,12 @@ void list_mini_icon(unsigned long *body)
     t = t->next;
   if (t)
   {
-    t->mini_icon.width	 = mip->width;
-    t->mini_icon.height	 = mip->height;
-    t->mini_icon.depth	 = mip->depth;
+    t->mini_icon.width   = mip->width;
+    t->mini_icon.height  = mip->height;
+    t->mini_icon.depth   = mip->depth;
     t->mini_icon.picture = mip->picture;
-    t->mini_icon.mask	 = mip->mask;
-    t->mini_icon.alpha	 = mip->alpha;
+    t->mini_icon.mask    = mip->mask;
+    t->mini_icon.alpha   = mip->alpha;
     /* repaint by clearing window */
     if (MiniIcons && t->mini_icon.picture) {
       if (t->PagerView)
@@ -1325,7 +1324,7 @@ void list_restack(unsigned long *body, unsigned long length)
 /***********************************************************************
  *
  *  Procedure:
- *	list_end - displays packet contents to stderr
+ *      list_end - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_end(void)
@@ -1396,32 +1395,18 @@ void list_config_info(unsigned long *body)
     {
       val = val - desk1;
     }
-    DrawGrid(val, True);
+    DrawGrid(val, True, None, NULL);
   }
 }
 
 void list_property_change(unsigned long *body)
 {
-  int i;
-
   if (body[0] == MX_PROPERTY_CHANGE_BACKGROUND)
   {
 
-    if (default_pixmap == ParentRelative &&
-	((!Swallowed && body[2] == 0) || (Swallowed && body[2] == Scr.Pager_w)))
+    if (((!Swallowed && body[2] == 0) || (Swallowed && body[2] == Scr.Pager_w)))
     {
-      ReConfigure();
-    }
-    if (BalloonView != None)
-    {
-      for(i=0;i<ndesks;i++)
-      {
-	if (Desks[i].ballooncolorset > -1 &&
-	    Colorset[Desks[i].ballooncolorset].pixmap == ParentRelative)
-	{
-	  XClearArea(dpy, Desks[i].balloon.w, 0, 0, 0, 0, True);
-	}
-      }
+	    update_pr_transparent_windows();
     }
   }
   else if  (body[0] == MX_PROPERTY_CHANGE_SWALLOW && body[2] == Scr.Pager_w)
@@ -1440,9 +1425,9 @@ int My_XNextEvent(Display *dpy, XEvent *event)
   fd_set in_fdset;
   static int miss_counter = 0;
 
-  if(XPending(dpy))
+  if(FPending(dpy))
   {
-    XNextEvent(dpy,event);
+    FNextEvent(dpy,event);
     return 1;
   }
 
@@ -1454,9 +1439,9 @@ int My_XNextEvent(Display *dpy, XEvent *event)
   {
   if(FD_ISSET(x_fd, &in_fdset))
     {
-      if(XPending(dpy))
+      if(FPending(dpy))
 	{
-	  XNextEvent(dpy,event);
+	  FNextEvent(dpy,event);
 	  miss_counter = 0;
 	  return 1;
 	}
@@ -1594,7 +1579,7 @@ void ParseOptions(void)
   int desk;
   int dx = 3;
   int dy = 3;
-
+  FvwmPictureAttributes fpa;
   Scr.FvwmRoot = NULL;
   Scr.Hilite = NULL;
   Scr.VScale = 32;
@@ -1602,6 +1587,11 @@ void ParseOptions(void)
   Scr.MyDisplayWidth = DisplayWidth(dpy, Scr.screen);
   Scr.MyDisplayHeight = DisplayHeight(dpy, Scr.screen);
 
+  fpa.mask = 0;
+  if (Pdepth <= 8)
+  {
+	  fpa.mask |= FPAM_DITHER;
+  }
   InitGetConfigLine(fd,CatString3("*",MyName,0));
   for (GetConfigLine(fd,&tline); tline != NULL; GetConfigLine(fd,&tline))
   {
@@ -1897,17 +1887,15 @@ void ParseOptions(void)
 	    PDestroyFvwmPicture(dpy, item->next->bgPixmap);
 	    item->next->bgPixmap = NULL;
 	  }
-	  item->next->bgPixmap = PCacheFvwmPicture (dpy, Scr.Pager_w,
-						    ImagePath,
-						    arg2, 0);
+	  item->next->bgPixmap = PCacheFvwmPicture(
+		  dpy, Scr.Pager_w, ImagePath, arg2, fpa);
 	}
 	else
 	{
 	  /* new Dcolor and desktop */
 	  item = NewPagerStringItem(item, desk);
-	  item->bgPixmap = PCacheFvwmPicture (dpy, Scr.Pager_w,
-					      ImagePath,
-					      arg2, 0);
+	  item->bgPixmap = PCacheFvwmPicture(
+		  dpy, Scr.Pager_w, ImagePath, arg2, fpa);
 	}
 	if (desk == Scr.CurrentDesk)
 	{
@@ -1917,9 +1905,8 @@ void ParseOptions(void)
 	    Desks[0].bgPixmap = NULL;
 	  }
 
-	  Desks[0].bgPixmap = PCacheFvwmPicture (dpy, Scr.Pager_w,
-					    ImagePath,
-					    arg2, 0);
+	  Desks[0].bgPixmap = PCacheFvwmPicture(
+		  dpy, Scr.Pager_w, ImagePath, arg2, fpa);
 	}
       }
       else if((desk >= desk1)&&(desk <=desk2))
@@ -1931,9 +1918,8 @@ void ParseOptions(void)
 	  PDestroyFvwmPicture(dpy, Desks[dNr].bgPixmap);
 	  Desks[dNr].bgPixmap = NULL;
 	}
-	Desks[dNr].bgPixmap = PCacheFvwmPicture (dpy, Scr.Pager_w,
-						 ImagePath,
-						 arg2, 0);
+	Desks[dNr].bgPixmap = PCacheFvwmPicture(
+		dpy, Scr.Pager_w, ImagePath, arg2, fpa);
       }
 
 #ifdef DEBUG
@@ -1951,9 +1937,8 @@ void ParseOptions(void)
 	  PixmapBack = NULL;
 	}
 
-	PixmapBack = PCacheFvwmPicture (dpy, Scr.Pager_w,
-					ImagePath,
-					arg1, 0);
+	PixmapBack = PCacheFvwmPicture(
+		dpy, Scr.Pager_w, ImagePath, arg1, fpa);
 #ifdef DEBUG
 	fprintf(stderr,
 		"[ParseOptions]: Global: bgPixmap = %s\n", arg1);
@@ -1970,9 +1955,8 @@ void ParseOptions(void)
 	  HilightPixmap = NULL;
 	}
 
-	HilightPixmap = PCacheFvwmPicture (dpy, Scr.Pager_w,
-					ImagePath,
-					arg1, 0);
+	HilightPixmap = PCacheFvwmPicture (
+		dpy, Scr.Pager_w, ImagePath, arg1, fpa);
 
 #ifdef DEBUG
 	fprintf(stderr,
@@ -2184,6 +2168,17 @@ void ParseOptions(void)
     else if (StrEquals(resource, "BalloonYOffset"))
     {
       sscanf(arg1, "%d", &BalloonYOffset);
+      if (BalloonYOffset == 0)
+      {
+	fprintf(stderr,
+		"%s: Warning:"
+		" you're not allowed BalloonYOffset 0; defaulting to +3\n",
+		MyName);
+	/* we don't allow yoffset of 0 because it allows direct transit from
+	 * pager window to balloon window, setting up a LeaveNotify/EnterNotify
+	 * event loop */
+	BalloonYOffset = 3;
+      }
     }
     else if (StrEquals(resource,"BalloonStringFormat"))
     {
