@@ -5,12 +5,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "Tools.h"
@@ -20,7 +20,7 @@
 
 /***********************************************/
 /***********************************************/
-/* Fonction pour PushButton		       */
+/* Fonction pour PushButton                    */
 /***********************************************/
 /***********************************************/
 void InitPushButton(struct XObj *xobj)
@@ -144,11 +144,11 @@ void EvtMousePushButton(struct XObj *xobj, XButtonEvent *EvtButton)
 
     while (End)
     {
-      XNextEvent(dpy, &event);
+      FNextEvent(dpy, &event);
       switch (event.type)
       {
       case EnterNotify:
-	XQueryPointer(dpy, *xobj->ParentWin,
+	FQueryPointer(dpy, *xobj->ParentWin,
 		      &Win1, &Win2, &x1, &y1, &x2, &y2, &modif);
 	if (WinBut == 0)
 	{
@@ -174,7 +174,7 @@ void EvtMousePushButton(struct XObj *xobj, XButtonEvent *EvtButton)
 	}
 	break;
       case LeaveNotify:
-	XQueryPointer(dpy, *xobj->ParentWin,
+	FQueryPointer(dpy, *xobj->ParentWin,
 		      &Win1, &Win2, &x1, &y1, &x2, &y2, &modif);
 	if (Win2 == WinBut)
 	{
@@ -239,12 +239,12 @@ void EvtMousePushButton(struct XObj *xobj, XButtonEvent *EvtButton)
       mask = 0;
       Attr.background_pixel = xobj->TabColor[back];
       mask |= CWBackPixel;
-      Attr.border_pixel	 =  0;
+      Attr.border_pixel  =  0;
       mask |= CWBorderPixel;
       Attr.colormap = Pcmap;
       mask |= CWColormap;
       Attr.cursor = XCreateFontCursor(dpy, XC_hand2);
-      mask |= CWCursor;		/* Curseur pour la fenetre */
+      mask |= CWCursor;         /* Curseur pour la fenetre */
       Attr.override_redirect = True;
       mask |= CWOverrideRedirect;
       WinPop = XCreateWindow(dpy, Root, x, y, wMenu-5, hMenu, 0,
@@ -259,7 +259,7 @@ void EvtMousePushButton(struct XObj *xobj, XButtonEvent *EvtButton)
       DrawPMenu(xobj, WinPop, hOpt, 1);
       do
       {
-	XQueryPointer(dpy, Root, &Win1, &Win2, &x1, &y1, &x2, &y2, &modif);
+	FQueryPointer(dpy, Root, &Win1, &Win2, &x1, &y1, &x2, &y2, &modif);
 	/* Determiner l'option courante / Current option */
 	y2 = y2 - y;
 	x2 = x2 - x;
@@ -279,7 +279,7 @@ void EvtMousePushButton(struct XObj *xobj, XButtonEvent *EvtButton)
 	  }
 	}
       }
-      while (!XCheckTypedEvent(dpy, ButtonRelease, &event));
+      while (!FCheckTypedEvent(dpy, ButtonRelease, &event));
       XDestroyWindow(dpy, WinPop);
       if (newvalue != 0)
       {
