@@ -78,7 +78,7 @@ extern int NRows, RowHeight;
 void Draw3dRect(Window wn, int x, int y, int w, int h, int state,
 		Bool iconified)
 {
-  colorset_struct *cset = &Colorset[0];  /* initialize to avoid warning */
+  colorset_t *cset = &Colorset[0];  /* initialize to avoid warning */
   int d = 1;
 
   if (ThreeDfvwm)
@@ -273,10 +273,17 @@ void ButtonDraw(Button *button, int x, int y, int w, int h)
 		int offset = (button->p.height > h) ?
 			0 : ((h - button->p.height) >> 1);
 		int pwidth = min(button->p.width, w-5);
+		FvwmRenderAttributes fra;
 
-		PGraphicsCopyFvwmPicture(dpy, &(button->p), win, hilite,
-					 0, 0, pwidth, pheight,
-					 x+3, y+offset);
+		fra.mask = FRAM_DEST_IS_A_WINDOW;
+		if (cs >= 0)
+		{
+			fra.mask |= FRAM_HAVE_ICON_CSET;
+			fra.colorset = &Colorset[cs];
+		}
+		PGraphicsRenderPicture(
+			dpy, win, &(button->p), &fra, win, *drawgc, None, None,
+			0, 0, pwidth, pheight, x+3, y+offset, 0, 0, False);
 		newx += pwidth+2;
 	}
 
@@ -673,7 +680,7 @@ int LocateButton(ButtonArray *array, int xp,  int yp, int *xb, int *yb,
 /* -------------------------------------------------------------------------
     ButtonCoordinates - Compute the coordinates of a button (animation)
    ------------------------------------------------------------------------- */
-extern int StartButtonWidth;
+extern int  StartAndLaunchButtonsWidth;
 void ButtonCoordinates(ButtonArray *array, int numbut, int *xc, int *yc)
 {
 	Button *temp;
@@ -695,7 +702,7 @@ void ButtonCoordinates(ButtonArray *array, int numbut, int *xc, int *yc)
 		}
 	}
 
-	*xc = x+StartButtonWidth+3;
+	*xc = x+ StartAndLaunchButtonsWidth+3;
 	*yc = y;
 }
 
