@@ -170,6 +170,9 @@ static volatile sig_atomic_t stick_taskbar_alarm = False;
 
 int UpdateInterval = 30;
 
+int whichButton = -1; 
+Bool startButtonPressed = FALSE;
+
 ButtonArray buttons;
 List windows;
 
@@ -215,6 +218,7 @@ rectangle screen_g;
 rectangle global_scr_g;
 
 int NRows, RowHeight, Midline;
+int ButtonPressed;
 
 #define COUNT_LIMIT    10000
 long DeskNumber;               /* Added by Balaji R */
@@ -226,10 +230,11 @@ extern int stwin_width, goodies_width, goodies_fontheight;
 extern TipStruct Tip;
 
 /* Imported from Start */
-extern int StartButtonWidth, StartButtonHeight;
-extern char *StartPopup;
-extern char *StartCommand;
-
+extern int StartAndLaunchButtonsWidth, StartAndLaunchButtonsHeight;
+extern StartAndLaunchButtonItem *First_Start_Button;
+extern StartAndLaunchButtonItem *Last_Start_Button;
+extern int WindowButtonsLeftMargin;
+extern int WindowButtonsRightMargin;
 char *ImagePath = NULL;
 char *XineramaConfig = NULL;
 static int fscreen = 0;
@@ -360,9 +365,9 @@ int main(int argc, char **argv)
   StartButtonInit(RowHeight);
 
   /* init the array of buttons */
-  InitArray(&buttons, StartButtonWidth + 4, 0,
-		      win_width - stwin_width - 8 - StartButtonWidth -10,
-		      RowHeight, button_width);
+  InitArray(&buttons, StartAndLaunchButtonsWidth + WindowButtonsLeftMargin, 0,
+	    win_width - stwin_width - StartAndLaunchButtonsWidth - WindowButtonsLeftMargin
+	    - WindowButtonsRightMargin, RowHeight, button_width);
   InitList(&windows);
 
   /* Request a list of all windows,
@@ -416,7 +421,7 @@ void EndLessLoop(void)
     if ( fvwmSelect(fd_width, &readset, NULL, NULL, &tv) > 0 )
     {
 
-      if (FD_ISSET(x_fd, &readset) || XPending(dpy))
+      if (FD_ISSET(x_fd, &readset) || FPending(dpy))
         LoopOnEvents();
 
       if (FD_ISSET(Fvwm_fd[1], &readset))
@@ -559,7 +564,9 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	  XResizeWindow(dpy, win, win_width, win_height);
 
 	UpdateArray(&buttons, -1, -1,
-		    win_width - stwin_width - 8 - StartButtonWidth -10,-1, -1);
+		    win_width - stwin_width - StartAndLaunchButtonsWidth
+		    - WindowButtonsLeftMargin - WindowButtonsRightMargin,
+		    -1, -1);
 	ArrangeButtonArray (&buttons);
       }
       if (AutoStick && win_is_shaded != IS_SHADED(cfgpacket))
@@ -933,7 +940,7 @@ void WaitForExpose(void)
   XEvent Event;
 
   while(1) {
-    XNextEvent(dpy, &Event);
+    FNextEvent(dpy, &Event);
     if (Event.type == Expose) {
       if (Event.xexpose.count == 0) break;
     }
@@ -964,7 +971,7 @@ void RedrawWindow(int force)
   }
   DrawButtonArray(&buttons, force);
   StartButtonDraw(force);
-  if (XQLength(dpy) && !force)
+  if (FQLength(dpy) && !force)
     LoopOnEvents();
 }
 
@@ -1247,7 +1254,7 @@ DoAlarmAction(void)
       /* We are now "sure" that the TaskBar is not hidden for the
 	 Event loop. We send a motion notify for activating tips */
       WindowState = 1;
-      if (XQueryPointer(dpy, win, &dummy_rt,&dummy_c, &abs_x, &abs_y,
+      if (FQueryPointer(dpy, win, &dummy_rt,&dummy_c, &abs_x, &abs_y,
 			&pos_x, &pos_y, &dummy) == False)
       {
 	/* pointer is on a different screen */
@@ -1258,7 +1265,7 @@ DoAlarmAction(void)
       sevent.xmotion.y = pos_y;
       sevent.xany.type = MotionNotify;
       sevent.xmotion.state = 0;
-      XSendEvent(dpy, win, False, EnterWindowMask, &sevent);
+      FSendEvent(dpy, win, False, EnterWindowMask, &sevent);
       Tip.type = NO_TIP;
     }
     else
@@ -1274,9 +1281,14 @@ void CheckForTip(int x, int y)
   int  num, bx, by, trunc;
   char *name;
 
-  if (MouseInStartButton(x, y)) {
-    if (Tip.type != START_TIP) PopupTipWindow(3, 0, "Click here to start");
-    Tip.type = START_TIP;
+  if (MouseInStartButton(x, y, &whichButton, &startButtonPressed)) {
+    if((!whichButton) && (First_Start_Button->isStartButton))
+    {
+      if (Tip.type != START_TIP) PopupTipWindow(3, 0, "Click here to start");
+      Tip.type = START_TIP;
+    }
+    else
+      Tip.type = NO_TIP;
   }
   else if (MouseInMail(x, y)) {
     if (Tip.type != MAIL_TIP) CreateMailTipWindow();
@@ -1321,14 +1333,14 @@ void LoopOnEvents(void)
   static unsigned long lasttime = 0L;
   Time NewTimestamp = lasttime;
 
-  while(XPending(dpy))
+  while(FPending(dpy))
   {
     redraw = -1;
-    XNextEvent(dpy, &Event);
+    FNextEvent(dpy, &Event);
     if (Event.xany.type == ConfigureNotify)
     {
       /* Purge all but the last configure events */
-      while (XCheckTypedEvent(dpy, ConfigureNotify, &Event))
+      while (FCheckTypedEvent(dpy, ConfigureNotify, &Event))
 	;
     }
 
@@ -1338,10 +1350,45 @@ void LoopOnEvents(void)
     case ButtonRelease:
       NewTimestamp = Event.xbutton.time;
       num = WhichButton(&buttons, Event.xbutton.x, Event.xbutton.y);
-      if (num == -1) {
-	if (MouseInStartButton(Event.xbutton.x, Event.xbutton.y))
-	  StartButtonUpdate(NULL, BUTTON_UP);
-      } else {
+      StartButtonUpdate(NULL, -1, BUTTON_UP);
+      if (num == -1) 
+      {
+	if (MouseInStartButton(Event.xbutton.x, Event.xbutton.y, &whichButton, &startButtonPressed))
+	{
+	  if (whichButton == ButtonPressed)
+	  {
+	    if ((First_Start_Button->buttonStartCommand != NULL) && (startButtonPressed))
+	    {
+	      rectangle r;
+	      Window tmpw;
+	      r.x = 0;
+	      r.y = 0;
+	      r.width = StartAndLaunchButtonsWidth;
+	      r.height = StartAndLaunchButtonsHeight;
+	      XTranslateCoordinates(dpy, win, Root, r.x, r.y, &r.x, &r.y, &tmpw);
+	      tmp = module_expand_action(dpy, screen, First_Start_Button->buttonStartCommand, &r, NULL, NULL);  
+	      if (tmp)
+	      {
+		SendText(Fvwm_fd, tmp, 0);
+		free(tmp);
+	      }
+	      else
+	      {
+		SendText(Fvwm_fd, First_Start_Button->buttonStartCommand, 0);
+	      }
+	    }
+	    else
+	    {
+	      tmp = (char *)safemalloc(100 * sizeof(char)); /* fix this later */
+	      getButtonCommand(whichButton, tmp, Event.xbutton.button);
+	      SendText(Fvwm_fd, tmp, 0);
+	      free(tmp);
+	    }
+	  }
+	}
+      } 
+      else 
+      {
 	ButReleased = ButPressed; /* Avoid race fvwm pipe */
 	if (Event.xbutton.button >= 1 &&
 	    Event.xbutton.button <= NUMBER_OF_MOUSE_BUTTONS)
@@ -1369,8 +1416,7 @@ void LoopOnEvents(void)
 	}
       }
 
-      if (MouseInStartButton(Event.xbutton.x, Event.xbutton.y)) {
-	StartButtonUpdate(NULL, BUTTON_UP);
+      if (MouseInStartButton(Event.xbutton.x, Event.xbutton.y, &whichButton, &startButtonPressed)) {
 	redraw = 0;
 	usleep(50000);
       }
@@ -1387,8 +1433,10 @@ void LoopOnEvents(void)
     case ButtonPress:
       NewTimestamp = Event.xbutton.time;
       RadioButton(&buttons, -1, BUTTON_UP); /* no windows focused anymore */
-      if (MouseInStartButton(Event.xbutton.x, Event.xbutton.y)) {
-	StartButtonUpdate(NULL, BUTTON_DOWN);
+      if (MouseInStartButton(Event.xbutton.x, Event.xbutton.y, &whichButton, &startButtonPressed))
+      {
+	StartButtonUpdate(NULL, whichButton, BUTTON_DOWN);
+	ButtonPressed = whichButton;
 	x = win_x;
 	if (win_y < Midline) {
 	  /* bar in top half of the screen */
@@ -1397,40 +1445,8 @@ void LoopOnEvents(void)
 	  /* bar in bottom of the screen */
 	  y = win_y - screen_g.height;
 	}
-	if (StartCommand != NULL)
-	{
-	  rectangle r;
-	  Window tmpw;
-
-	  r.x = 0;
-	  r.y = 0;
-	  r.width = StartButtonWidth;
-	  r.height = StartButtonHeight;
-	  XTranslateCoordinates(dpy, win, Root, r.x, r.y, &r.x, &r.y, &tmpw);
-	  tmp = module_expand_action(dpy, screen, StartCommand, &r, NULL, NULL);
-	  if (tmp)
-	  {
-	    SendText(Fvwm_fd, tmp, win);
-	    free(tmp);
-	  }
-	  else
-	  {
-	    SendText(Fvwm_fd, StartCommand, win);
-	  }
-	}
-	if (StartPopup != NULL)
-	{
-	  tmp = (char *)safemalloc(strlen(StartPopup) + 7);
-	  sprintf(tmp,"Popup %s", StartPopup);
-	  SendText(Fvwm_fd, tmp, win);
-	  free(tmp);
-	}
-	if (StartPopup == NULL && StartCommand == NULL)
-	{
-	  SendText(Fvwm_fd, "Popup StartMenu", win);
-	}
       } else {
-	StartButtonUpdate(NULL, BUTTON_UP);
+	StartButtonUpdate(NULL, whichButton, BUTTON_UP);
 	if (MouseInMail(Event.xbutton.x, Event.xbutton.y)) {
 	  HandleMailClick(Event);
 	} else {
@@ -1447,7 +1463,7 @@ void LoopOnEvents(void)
     case Expose:
       memcpy(&Event2, &Event, sizeof(Event));
       /* eat up excess Expose events. */
-      while (XCheckTypedWindowEvent(dpy, win, Expose, &Event2))
+      while (FCheckTypedWindowEvent(dpy, win, Expose, &Event2))
       {
 	memcpy(&Event, &Event2, sizeof(Event));
       }
@@ -1522,13 +1538,10 @@ void LoopOnEvents(void)
 
     case MotionNotify:
       NewTimestamp = Event.xmotion.time;
-      if (MouseInStartButton(Event.xmotion.x, Event.xbutton.y)) {
-	if (SomeButtonDown(Event.xmotion.state))
-	  redraw = StartButtonUpdate(NULL, BUTTON_DOWN) ? 0 : -1;
+      if (MouseInStartButton(Event.xmotion.x, Event.xbutton.y, &whichButton, &startButtonPressed)) {
 	CheckForTip(Event.xmotion.x, Event.xmotion.y);
 	break;
       }
-      redraw = StartButtonUpdate(NULL, BUTTON_UP) ? 0 : -1;
       num = WhichButton(&buttons, Event.xmotion.x, Event.xmotion.y);
       if (!HighlightFocus) {
 	if (SomeButtonDown(Event.xmotion.state) && num != ButPressed) {
@@ -1550,7 +1563,7 @@ void LoopOnEvents(void)
     case ConfigureNotify:
       memcpy(&Event2, &Event, sizeof(Event));
       /* eat up excess ConfigureNotify events. */
-      while (XCheckTypedWindowEvent(dpy, win, ConfigureNotify, &Event2))
+      while (FCheckTypedWindowEvent(dpy, win, ConfigureNotify, &Event2))
       {
 	memcpy(&Event, &Event2, sizeof(Event));
       }
@@ -1797,13 +1810,13 @@ static void CreateOrUpdateGCs(void)
    else
      hilite = fvwmlib_XCreateGC(dpy,win,gcmask,&gcval);
 
-   gcval.foreground = WhitePixel(dpy, screen);;
+   gcval.foreground = PictureWhitePixel();
    if (whitegc)
      XChangeGC(dpy,whitegc,gcmask,&gcval);
    else
      whitegc = fvwmlib_XCreateGC(dpy,win,gcmask,&gcval);
 
-   gcval.foreground = BlackPixel(dpy, screen);
+   gcval.foreground = PictureBlackPixel();
    if (blackgc)
      XChangeGC(dpy,blackgc,gcmask,&gcval);
    else
@@ -1982,12 +1995,12 @@ void StartMeUp(void)
    win_y = hints.y;
 
    if(Pdepth < 2) {
-     back = WhitePixel(dpy, screen);
-     fore = BlackPixel(dpy, screen);
-     iconback = WhitePixel(dpy, screen);
-     iconfore = BlackPixel(dpy, screen);
-     focusback = WhitePixel(dpy, screen);
-     focusfore = BlackPixel(dpy, screen);
+     back = PictureWhitePixel();
+     fore = PictureBlackPixel();
+     iconback = PictureWhitePixel();
+     iconfore = PictureBlackPixel();
+     focusback = PictureWhitePixel();
+     focusfore = PictureBlackPixel();
    } else {
      back = GetColor(BackColor);
      fore = GetColor(ForeColor);
@@ -2234,7 +2247,7 @@ void HideTaskBar()
 
   if (FocusInWin)
   {
-    if (XQueryPointer(
+    if (FQueryPointer(
 		dpy, win, &d_rt,&d_ch, &d_x, &d_y, &wx, &wy, &mask) == False)
     {
       /* pointer is on a different screen - that's okay here */
